unit UCaptureKey;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, IniFiles, UComPort;

type
  TCaptureKeyForm = class(TForm)
    ComPortMemo: TMemo;
    NameEdt: TLabeledEdit;
    CodeEdt: TLabeledEdit;
    RepeatCodeEdt: TLabeledEdit;
    CaptureBtn: TButton;
    OkBtn: TButton;
    CancelBtn: TButton;
    GenericRepeatCodeCb: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure CaptureBtnClick(Sender: TObject);
    procedure ComPortRxChar(Sender: TObject; Count: Integer);
    procedure OkBtnClick(Sender: TObject);
  private
    FCapturing: Boolean;
  public
    procedure ApplyLanguage(Lang: string);
  end;

var
  CaptureKeyForm: TCaptureKeyForm;

implementation

uses UMain;

{$R *.dfm}

//------------------------------------------------------------------------------

procedure TCaptureKeyForm.ApplyLanguage(Lang: string);
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.lang')) do
  try
    Self.Caption := ReadString(Lang, 'CustomKeyTitle', 'Define custom key');
    NameEdt.EditLabel.Caption := ReadString(Lang, 'NameCaption', 'Name :');
    GenericRepeatCodeCb.Caption := ReadString(Lang, 'GenRepeatCodeCaption', 'Generic repeat code');
    CaptureBtn.Caption := ReadString(Lang, 'CaptureBtnCaption', 'Capture');
    CodeEdt.EditLabel.Caption := ReadString(Lang, 'CodeCaption', 'Code :');
    RepeatCodeEdt.EditLabel.Caption := ReadString(Lang, 'RepeatCodeCaption', 'Repeat code :');
    OkBtn.Caption := ReadString(Lang, 'OkBtnCaption', 'Ok');
    CancelBtn.Caption := ReadString(Lang, 'CancelBtnCaption', 'Cancel');
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TCaptureKeyForm.FormCreate(Sender: TObject);
begin
  ClientHeight := 240;
  ClientWidth := 530;
  FCapturing := False;
  ApplyLanguage(CurrentLanguageStr);
end;

//------------------------------------------------------------------------------

procedure TCaptureKeyForm.CaptureBtnClick(Sender: TObject);
begin
  if FCapturing then
  begin
    FCapturing := False;
    OkBtn.Enabled := True;
    CancelBtn.Enabled := True;
    CaptureBtn.Caption := 'Capture';
    CodeEdt.ReadOnly := False;
    RepeatCodeEdt.ReadOnly := False;
  end
  else begin
    FCapturing := True;
    OkBtn.Enabled := False;
    CancelBtn.Enabled := False;
    CaptureBtn.Caption := 'Cancel';
    CodeEdt.ReadOnly := True;
    CodeEdt.Text := '';
    RepeatCodeEdt.ReadOnly := True;
    RepeatCodeEdt.Text := '';
  end;
end;

//------------------------------------------------------------------------------

procedure TCaptureKeyForm.OkBtnClick(Sender: TObject);
begin
  if not FCapturing then
    if NameEdt.Text <> '' then
      if CodeEdt.Text <> '' then
        ModalResult := mrOk
      else ComPortMemo.Lines.Add('KEY CODE MISSING')
    else ComPortMemo.Lines.Add('KEY NAME MISSING')
  else ComPortMemo.Lines.Add('IN CAPTURE MODE');
end;

//------------------------------------------------------------------------------

procedure TCaptureKeyForm.ComPortRxChar(Sender: TObject; Count: Integer);
var
  I, Rd: Integer;
  Buf: array[0..1023] of Byte;
  KeyCode: string;
begin
  while ComPortMemo.Lines.Count > 9 do
    ComPortMemo.Lines.Delete(0);
  Rd := (Sender as TComPort).Read(Buf, Count);
  KeyCode := '';
  for I := 0 to Rd - 1 do
    KeyCode := KeyCode + IntToHex(Buf[I], 2);

  if (KeyCode <> '') and (KeyCode <> '00') then
    if FCapturing then
      if CodeEdt.Text = '' then
      begin
        CodeEdt.Text := KeyCode;
        ComPortMemo.Lines.Add('''' + KeyCode + ''' => code.');
      end
      else if RepeatCodeEdt.Text = '' then
        if GenericRepeatCodeCb.Checked then
        begin
          RepeatCodeEdt.Text := CodeEdt.Text + KeyCode;
          ComPortMemo.Lines.Add('''' + CodeEdt.Text + KeyCode + ''' => repeat code.');
          CaptureBtnClick(nil);
        end
        else begin
          RepeatCodeEdt.Text := KeyCode;
          ComPortMemo.Lines.Add('''' + KeyCode + ''' => repeat code.');
          CaptureBtnClick(nil);
        end
      else ComPortMemo.Lines.Add(' ' + KeyCode)
    else ComPortMemo.Lines.Add(' ' + KeyCode);
end;

//------------------------------------------------------------------------------

end.

