unit UConfigKeys;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, StdCtrls, jpeg, ExtCtrls, Buttons, IniFiles, ImgList, Math,
  ShellAPI, UMain, UFunction, UAction, UKeybEmul, UComPort, UCaptureKey, XPMan,
  UGradPnl;

type
  PPhysicalKey = ^TPhysicalKey;
  TPhysicalKey = record
    Name: string;
    Code: string;
    RepeatCode: string;
    GenericRepeatCode: Boolean;
    Rect: TRect;
  end;

  PRemote = ^TRemote;
  TRemote = record
    Name: string;
    JPEGStr: TFileName;
    JPEG: TJPEGImage;
    Keys: TList;
  end;

  TConfigKeysForm = class(TForm)
    ActionList: TActionList;
    AddKey: TAction;
    RemoveKey: TAction;
    AddFunction: TAction;
    RemoveFunction: TAction;
    ModifyFunction: TAction;
    AddAction: TAction;
    RemoveAction: TAction;
    ModifyAction: TAction;
    Save: TAction;
    MoveFunctionUp: TAction;
    MoveFunctionDown: TAction;
    MoveActionUp: TAction;
    MoveActionDown: TAction;
    IconList: TImageList;
    ModifyKey: TAction;
    XPManifest1: TXPManifest;
    Splitter1: TSplitter;
    RightPnl: TPanel;
    Label1: TLabel;
    FunctionsGb: TGroupBox;
    FunctionsLb: TListBox;
    AddFunctionBtn: TButton;
    RemoveFunctionBtn: TButton;
    ModifyFunctionBtn: TButton;
    MoveFunctionUpBtn: TBitBtn;
    MoveFunctionDownBtn: TBitBtn;
    OptionsGb: TGroupBox;
    OSDLbl: TLabel;
    OsdCombo: TComboBox;
    RepeatActionsCb: TCheckBox;
    ActionsGb: TGroupBox;
    RemoveActionBtn: TButton;
    ModifyActionBtn: TButton;
    AddActionBtn: TButton;
    ActionsLb: TListBox;
    MoveActionUpBtn: TBitBtn;
    MoveActionDownBtn: TBitBtn;
    OkBtn: TButton;
    ShowComPort: TCheckBox;
    CenterPnl: TPanel;
    RemotePb: TPaintBox;
    CustomKeyPnl: TPanel;
    CustomKeysLb: TListBox;
    AddKeyBtn: TButton;
    RemoveKeyBtn: TButton;
    ModifyKeyBtn: TButton;
    RemoteCombo: TComboBox;
    ComPortMemo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure RemotePbMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RemotePbPaint(Sender: TObject);
    procedure AddKeyExecute(Sender: TObject);
    procedure RemoveKeyExecute(Sender: TObject);
    procedure ModifyKeyExecute(Sender: TObject);
    procedure AddFunctionExecute(Sender: TObject);
    procedure RemoveFunctionExecute(Sender: TObject);
    procedure ModifyFunctionExecute(Sender: TObject);
    procedure AddActionExecute(Sender: TObject);
    procedure RemoveActionExecute(Sender: TObject);
    procedure ModifyActionExecute(Sender: TObject);
    procedure SaveExecute(Sender: TObject);
    procedure MoveFunctionUpExecute(Sender: TObject);
    procedure MoveFunctionDownExecute(Sender: TObject);
    procedure MoveActionUpExecute(Sender: TObject);
    procedure MoveActionDownExecute(Sender: TObject);
    procedure RemoteComboChange(Sender: TObject);
    procedure CustomKeysLbClick(Sender: TObject);
    procedure FunctionsLbClick(Sender: TObject);
    procedure FunctionsLbDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure OsdComboChange(Sender: TObject);
    procedure RepeatActionsCbClick(Sender: TObject);
    procedure ActionsLbClick(Sender: TObject);
    procedure ActionsLbDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ShowComPortClick(Sender: TObject);
    procedure ComPortRxChar(Sender: TObject; Count: Integer);
    procedure Label1Click(Sender: TObject);
  private
    FRemotes: TList;
    FCustomKeys: TList;
    FSelectedKey: PRemoteKey;
    FFunctionForm: TFunctionForm;
    FActionForm: TActionForm;
    FCaptureKeyForm: TCaptureKeyForm;
    FGradPnl: TgradientPanel;
    FCustomKeyStr: string;
    procedure LoadRemotes;
    procedure RefreshFunctions;
    procedure RefreshActions;
    procedure RefreshCustomKeys;
    procedure SetSelectedKey(Key: PRemoteKey);
    procedure EnableFunctionGb(Value: Boolean);
    procedure EnableOSDGb(Value: Boolean);
    procedure EnableActionGb(Value: Boolean);
  public
    procedure ApplyLanguage(Lang: string);
    property Remotes: TList read FRemotes; 
  end;

const
  REMOTE_IMG_WIDTH = 210;
  REMOTE_IMG_HEIGHT = 450;

var
  ConfigKeysForm: TConfigKeysForm;

function FunctionName(Func: PKeyFunction): string;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------

function FunctionName(Func: PKeyFunction): string;
var
  I: Integer;
  VarValue: string;
begin
  if (Func.Data.TargetTest = ttNoTest) and (Func.Data.VarName = '') then
    Result := 'Default'
  else begin
    if Func.Data.TargetTest <> ttNoTest then
      case Func.Data.TargetTest of
        ttActiveWindowIs: Result := 'if active window is [' + Func.Data.Target + ']';
        ttWindowOpen: Result := 'if window [' + Func.Data.Target + '] is open';
      end
    else Result := '';
    if Func.Data.VarName <> '' then
    begin
      Result := Result + '#';
      VarValue := '!!error!!';
      for I := 0 to Variables.Count - 1 do
        if Variables[I].Name = Func.Data.VarName then
          if Variables[I].Values.Count > Func.Data.VarValue then
            VarValue := Variables[I].Values[Func.Data.VarValue];
      Result := Result + 'if ' + Func.Data.VarName + ' = ' + VarValue;
    end;
  end;
end;

//------------------------------------------------------------------------------

function ActionName(Act: PSingleAction): string;
var
  I: Integer;
begin
  case Act.ActionType of
    atKeyb: with Act.KeybData^ do
      begin
        case TargetType of
          ttClass, ttForm: Result := '03{target = ' + Target + '} ';
          else Result := '03';
        end;
        if Alt then Result := Result + 'Alt + ';
        if Ctrl then Result := Result + 'Ctrl + ';
        if Shift then Result := Result + 'Shift + ';
        if Windows then Result := Result + 'Win + ';
        for I := 0 to NBKEYBKEYS - 1 do
          if KEYBKEYS[I].Ch = Ch then
          begin
            Result := Result + KEYBKEYS[I].Name;
            Break;
          end;
      end;
    atExec: with Act.ExecData^ do
        Result := '04' + ExtractFileName(Path);
    atAMix: with Act.AMixData^ do
      begin
        if Line > 0 then
          Result := '05' + AMixer.Destinations[Device].Connections[Line - 1].Data.szShortName
        else
          Result := '05' + AMixer.Destinations[Line].Data.szName;
        if Mute then
          Result := Result + ', Mute'
        else if Vol > 0 then
          Result := Result + ', +' + IntToStr(Vol)
        else
          Result := Result + ', - ' + IntToStr(Abs(Vol));
      end;
    atCVar: with Act.CVarData^ do
      begin
        if Loop = LOOP_UP then
          Result := '01' + VarName + ' := succ(' + VarName + ')'
        else if Loop = LOOP_DOWN then
          Result := '01' + VarName + ' := prev(' + VarName + ')'
        else
          for I := 0 to Variables.Count - 1 do
            if Variables[I].Name = VarName then
            begin
              Result := '01' + VarName + ' := ' + Variables[I].Values[SetValue];
              Break;
            end
      end;
    atMisc: with Act.MiscData^ do
        case Category of
          OSD_CAT: Result := '06OSD - ' + OSDOPT[Value];
          MORETV_CAT: Result := '08MoreTV - ' + MORETVOPT[Value];
          KTV_CAT: Result := '09K!TV - ' + KTVOPT[Value];
          WINAMP_CAT: Result := '07Winamp - ' + WINAMPOPT[Value];
          MEDIAPLAYER_CAT: Result := '10Windows Media Player - ' + WMPOPT[Value];
          BSPLAYER_CAT: Result := '11BSPlayer - ' + BSPLAYOPT[Value];
          POWERDVD_CAT: Result := '12Power DVD - ' + PDVDOPT[Value];
          WINSTB_CAT: Result := '13WinSTB - ' + WINSTBOPT[Value];
          //ANOTHERPROG_CAT: Result := '10AnotherProg - ' + WMPOPT[Value];
          else Result := '06!! Error !! reference doesn''t exist';
        end;
    else Result := '!! Error !!';
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.LoadRemotes;
var
  Folders: TStringList;
  Remote: PRemote;
  I: Integer;

  //-----------------------------------------
  function LoadRemote(Path: string): PRemote;
  var
    J: Integer;
    Remote: PRemote;
    PKey: PPhysicalKey;
  begin
    with TIniFile.Create(Path + '\remote.ini') do
    begin
      J := 0;
      New(Remote);
      FillChar(Remote^, SizeOf(TRemote), 0);
      Remote.Name := ReadString('Main', 'Name', 'Remote ' + IntToStr(I));
      Remote.JPEGStr := Path + '\' + ReadString('Main', 'Picture', 'Remote.jpg');
      Remote.JPEG := nil;
      Remote.Keys := TList.Create;
      while SectionExists('Key' + IntToStr(J)) do
      begin
        New(PKey);
        FillChar(PKey^, SizeOf(TPhysicalKey), 0);
        with PKey^ do
        begin
          Name := ReadString('Key' + IntToStr(J), 'Name', Remote.Name + ' - Key ' + IntToStr(J));
          Code := ReadString('Key' + IntToStr(J), 'Code', '');
          RepeatCode := ReadString('Key' + IntToStr(J), 'RepeatCode', '');
          GenericRepeatCode := ReadBool('Key' + IntToStr(J), 'GenericRepeatCode', False);
          Rect.Left := ReadInteger('Key' + IntToStr(J), 'Rect.Left', 0);
          Rect.Top := ReadInteger('Key' + IntToStr(J), 'Rect.Top', 0);
          Rect.Right := ReadInteger('Key' + IntToStr(J), 'Rect.Right', 0);
          Rect.Bottom := ReadInteger('Key' + IntToStr(J), 'Rect.Bottom', 0);
        end;
        Remote.Keys.Add(PKey);
        Inc(J);
      end;
    end;
    if FileExists(Remote.JPEGStr) then
      Result := Remote
    else begin
      Dispose(Remote);
      Result := nil;
    end;
  end;

  //---------------------------
  procedure AssignRemoteToKeys;
  var
    I, J, K: Integer;
    Found: Boolean;
  begin
    for I := 0 to Keys.Count - 1 do
    begin
      Keys[I].Remote := -1;
      Found := False;
      for J := 0 to FRemotes.Count - 1 do
      begin
        for K := 0 to PRemote(FRemotes[J]).Keys.Count - 1 do
          if PPhysicalKey(PRemote(FRemotes[J]).Keys[K]).Code = Keys[I].Data.Code then
          begin
            Keys[I].Remote := J;
            Found := True;
            Break;
          end;
        if Found then Break;
      end;
    end;
  end;

begin
  Folders := TStringList.Create;
  try
    GetDirList(ExtractFilePath(ParamStr(0)) + '*.*', Folders);
    for I := 0 to Folders.Count - 1 do
      if FileExists(ExtractFilePath(ParamStr(0)) + Folders[I] + '\remote.ini') then
      begin
        Remote := LoadRemote(ExtractFilePath(ParamStr(0)) + Folders[I]);
        if Remote <> nil then
          FRemotes.Add(Remote);
      end;
  finally
    Folders.Free;
  end;
  RemoteCombo.Items.Clear;
  for I := 0 to FRemotes.Count - 1 do
    RemoteCombo.Items.Add(PRemote(FRemotes[I]).Name);
  RemoteCombo.Items.Add(FCustomKeyStr);
  RemoteCombo.ItemIndex := 0;
  RemoteComboChange(RemoteCombo);
  AssignRemoteToKeys;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RefreshFunctions;
var
  I: Integer;
begin
  FunctionsLb.Clear;
  if FSelectedKey <> nil then
    for I := 0 to FSelectedKey.Functions.Count - 1 do
      FunctionsLb.Items.Add(FunctionName(FSelectedKey.Functions[I]));
  if FunctionsLb.Items.Count > 0 then
    FunctionsLb.ItemIndex := 0
  else FunctionsLb.ItemIndex := -1;
  FunctionsLbClick(nil);
  RefreshActions;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RefreshActions;
var
  I: Integer;
begin
  ActionsLb.Clear;
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) then
    for I := 0 to FSelectedKey.Functions[FunctionsLb.ItemIndex].Actions.Count - 1 do
      ActionsLb.Items.Add(ActionName(FSelectedKey.Functions[FunctionsLb.ItemIndex].Actions[I]));
  if ActionsLb.Items.Count > 0 then
    ActionsLb.ItemIndex := 0
  else ActionsLb.ItemIndex := -1;
  ActionsLbClick(nil);
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RefreshCustomKeys;
var
  I: Integer;
begin
  CustomKeysLb.Clear;
  FCustomKeys.Clear;
  for I := 0 to Keys.Count - 1 do
    if Keys[I].Remote = -1 then
    begin
      CustomKeysLb.Items.Add(Keys[I].Data.Name);
      FCustomKeys.Add(Keys[I]);
    end;
  if CustomKeysLb.Items.Count > 0 then
    CustomKeysLb.ItemIndex := 0
  else CustomKeysLb.ItemIndex := -1;
  CustomKeysLbClick(nil);
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.SetSelectedKey(Key: PRemoteKey);
begin
  FSelectedKey := Key;
  RefreshFunctions;
  EnableFunctionGb(Key <> nil);
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.EnableFunctionGb(Value: Boolean);
begin
  FunctionsLb.Enabled := Value;
  AddFunction.Enabled := Value;
  RemoveFunction.Enabled := Value;
  ModifyFunction.Enabled := Value;
  MoveFunctionUp.Enabled := Value;
  MoveFunctionDown.Enabled := Value;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.EnableOSDGb(Value: Boolean);
begin
  OSDLbl.Enabled := Value;
  OsdCombo.Enabled := Value;
  RepeatActionsCb.Enabled := Value;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.EnableActionGb(Value: Boolean);
begin
  ActionsLb.Enabled := Value;
  AddAction.Enabled := Value;
  RemoveAction.Enabled := Value;
  ModifyAction.Enabled := Value;
  MoveActionUp.Enabled := Value;
  MoveActionDown.Enabled := Value;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ApplyLanguage(Lang: string);
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.lang')) do
  try
    FGradPnl.Caption := ReadString(Lang, 'KeysTitle', 'Configuration - Keys');
    AddKeyBtn.Caption := ReadString(Lang, 'AddBtnCaption', 'Add...');
    AddFunctionBtn.Caption := ReadString(Lang, 'AddBtnCaption', 'Add...');
    AddActionBtn.Caption := ReadString(Lang, 'AddBtnCaption', 'Add...');
    RemoveKeyBtn.Caption := ReadString(Lang, 'RemoveBtnCaption', 'Remove');
    RemoveFunctionBtn.Caption := ReadString(Lang, 'RemoveBtnCaption', 'Remove');
    RemoveActionBtn.Caption := ReadString(Lang, 'RemoveBtnCaption', 'Remove');
    ModifyKeyBtn.Caption := ReadString(Lang, 'ModifyBtnCaption', 'Modify...');
    ModifyFunctionBtn.Caption := ReadString(Lang, 'ModifyBtnCaption', 'Modify...');
    ModifyActionBtn.Caption := ReadString(Lang, 'ModifyBtnCaption', 'Modify...');
    FunctionsGb.Caption := ReadString(Lang, 'FunctionsCaption', 'Functions');
    OptionsGb.Caption := ReadString(Lang, 'OptionsCaption', 'Options');
    OSDLbl.Caption := ReadString(Lang, 'OSDCaption', 'OSD :');
    RepeatActionsCb.Caption := ReadString(Lang, 'RepeatActionsCbCaption', 'Repeat actions');
    ActionsGb.Caption := ReadString(Lang, 'ActionsCaption', 'Actions');
    ShowComPort.Caption := ReadString(Lang, 'ShowCOMDataFlowCbCaption', 'Show COM data flow');
    FCustomKeyStr := ReadString(Lang, 'CustomKeysText', 'Custom keys');
    OkBtn.Caption := ReadString(Lang, 'OkBtnCaption', 'Ok');
    if RemoteCombo.Items.Count > 0 then
      RemoteCombo.Items[RemoteCombo.Items.Count - 1] := FCustomKeyStr;
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  ClientHeight := 525;
  ClientWidth := 645;
  Label1.Cursor := crHand;
  FGradPnl := TgradientPanel.Create(Self);
  with FGradPnl do
  begin
    Parent := RightPnl;
    Align := alTop;
    Caption := 'Configuration - Keys';
    Font.Name := 'Arial';
    Font.Size := 14;
    Font.Style := [fsBold];
    Font.Color := clWhite;
    FEndColor := clBtnFace;
    ;
    FBeginColor := clBlack;
  end;
  FRemotes := TList.Create;
  LoadRemotes;
  FCustomKeys := TList.Create;
  OsdCombo.Items.Clear;
  OsdCombo.Items.Add('Vol [vol_percent]%');
  OsdCombo.Items.Add('Vol [vol_gauge]');
  if AMixer.Destinations.Count > 0 then
    for I := 0 to AMixer.Destinations[0].Connections.Count - 1 do
      with AMixer.Destinations[0].Connections[I] do
      begin
        OsdCombo.Items.Add(Data.szName + ' [' + Data.szShortName + '_percent]%');
        OsdCombo.Items.Add(Data.szName + ' [' + Data.szShortName + '_gauge]');
      end;
  for I := 0 to Variables.Count - 1 do
    OsdCombo.Items.Add(Variables[I].Name + ' = [' + Variables[I].Name + '_value]');
  ApplyLanguage(CurrentLanguageStr);
  ComPort.OnRxChar := ComPortRxChar;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  //
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RemotePbMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  I, J: Integer;
  Found: Boolean;
  Key: PRemoteKey;
begin
  with PRemote(FRemotes[RemoteCombo.ItemIndex])^ do
    for I := 0 to Keys.Count - 1 do
      if PtInRect(PPhysicalKey(Keys[I]).Rect, Point(X, Y)) then
      begin
        with RemotePb.Canvas do
        begin
          Draw(0, 0, JPEG);
          Brush.Color := clFuchsia;
          FrameRect(PPhysicalKey(Keys[I]).Rect);
        end;
        Found := False;
        for J := 0 to UMain.Keys.Count - 1 do
          if UMain.Keys[J].Data.Code = PPhysicalKey(Keys[I]).Code then
          begin
            SetSelectedKey(UMain.Keys[J]);
            Found := True;
            Break;
          end;
        if not (Found) then
        begin
          New(Key);
          FillChar(Key^, SizeOf(TRemoteKey), 0);
          New(Key.Data);
          FillChar(Key.Data^, SizeOf(TRKeyData), 0);
          Key.Data.Name := PPhysicalKey(Keys[I]).Name;
          Key.Data.Code := PPhysicalKey(Keys[I]).Code;
          Key.Data.RepeatCode := PPhysicalKey(Keys[I]).RepeatCode;
          Key.Data.GenericRepeatCode := PPhysicalKey(Keys[I]).GenericRepeatCode;
          SetZarbCode(Key);
          Key.Remote := RemoteCombo.ItemIndex;
          Key.Functions := TFunctionList.Create;
          UMain.Keys.Add(Key);
          SetSelectedKey(Key);
        end;
        Break;
      end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RemotePbPaint(Sender: TObject);
var
  I: Integer;
begin
  with RemotePb.Canvas do
  begin
    Draw(0, 0, PRemote(FRemotes[RemoteCombo.ItemIndex]).JPEG);
    if FSelectedKey <> nil then
      for I := 0 to PRemote(FRemotes[RemoteCombo.ItemIndex]).Keys.Count - 1 do
        if FSelectedKey.Data.Code = PPhysicalKey(PRemote(FRemotes[RemoteCombo.ItemIndex]).Keys[I]).Code then
        begin
          Brush.Color := clFuchsia;
          FrameRect(PPhysicalKey(PRemote(FRemotes[RemoteCombo.ItemIndex]).Keys[I]).Rect);
          Break;
        end;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.AddKeyExecute(Sender: TObject);
var
  I: Integer;
  Key: PRemoteKey;
begin
  if FCaptureKeyForm = nil then
    FCaptureKeyForm := TCaptureKeyForm.Create(Self);
  ComPort.OnRxChar := FCaptureKeyForm.ComPortRxChar;
  with FCaptureKeyForm do
  begin
    NameEdt.SelectAll;
    if ShowModal = mrOk then
    begin
      New(Key);
      New(Key.Data);
      Key.Remote := -1;
      Key.Functions := TFunctionList.Create;
      Key.Data.Name := NameEdt.Text;
      Key.Data.Code := CodeEdt.Text;
      Key.Data.RepeatCode := RepeatCodeEdt.Text;
      Key.Data.GenericRepeatCode := GenericRepeatCodeCb.Checked;
      SetZarbCode(Key);
      Keys.Add(Key);
      I := CustomKeysLb.Items.Count;
      RefreshCustomKeys;
      CustomKeysLb.ItemIndex := I;
      CustomKeysLbClick(nil);
    end;
  end;
  ComPort.OnRxChar := ComPortRxChar;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RemoveKeyExecute(Sender: TObject);
begin
  if CustomKeysLb.ItemIndex > -1 then
  begin
    Keys.Remove(FCustomKeys[CustomKeysLb.ItemIndex]);
    RefreshCustomKeys;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ModifyKeyExecute(Sender: TObject);
var
  I: Integer;
  Key: PRemoteKey;
begin
  if FCaptureKeyForm = nil then
    FCaptureKeyForm := TCaptureKeyForm.Create(Self);
  ComPort.OnRxChar := FCaptureKeyForm.ComPortRxChar;
  with FCaptureKeyForm do
  begin
    Key := FCustomKeys[CustomKeysLb.ItemIndex];
    NameEdt.Text := Key.Data.Name;
    CodeEdt.Text := Key.Data.Code;
    RepeatCodeEdt.Text := Key.Data.RepeatCode;
    GenericRepeatCodeCb.Checked := Key.Data.GenericRepeatCode;
    NameEdt.SelectAll;
    if ShowModal = mrOk then
    begin
      Key.Remote := -1;
      Key.Data.Name := NameEdt.Text;
      Key.Data.Code := CodeEdt.Text;
      Key.Data.RepeatCode := RepeatCodeEdt.Text;
      Key.Data.GenericRepeatCode := GenericRepeatCodeCb.Checked;
      SetZarbCode(Key);
      I := CustomKeysLb.ItemIndex;
      RefreshCustomKeys;
      CustomKeysLb.ItemIndex := I;
      CustomKeysLbClick(nil);
    end;
  end;
  ComPort.OnRxChar := ComPortRxChar;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.AddFunctionExecute(Sender: TObject);
var
  I: Integer;
  Func: PKeyFunction;
begin
  if FSelectedKey <> nil then
  begin
    if FFunctionForm = nil then
      FFunctionForm := TFunctionForm.Create(self);
    with FFunctionForm do
      if ShowModal = mrOk then
      begin
        New(Func);
        FillChar(Func^, SizeOf(TKeyFunction), 0);
        New(Func.Data);
        FillChar(Func.Data^, SizeOf(TFuncData), 0);
        with Func.Data^ do
        begin
          if TestRb.Checked and WindowCb.Checked then
          begin
            TargetTest := TTargetTest(TargetTestCombo.ItemIndex + 1);
            if ClassRb.Checked then
            begin
              Target := ClassCombo.Text;
              TargetType := ttClass;
            end
            else begin
              Target := FormCombo.Text;
              TargetType := ttForm;
            end;
          end
          else begin
            TargetTest := ttNoTest;
            Target := '';
            TargetType := ttNone;
          end;
          if TestRb.Checked and VariableCb.Checked then
          begin
            VarName := VarNameCombo.Items[VarNameCombo.ItemIndex];
            VarValue := VarValueCombo.ItemIndex;
          end
          else begin
            VarName := '';
            VarValue := -1;
          end;
          OSD := '';
          RepeatActions := False;
        end;
        Func.Actions := TSingleActionList.Create;
        I := FSelectedKey.Functions.Add(Func);
        RefreshFunctions;
        FunctionsLb.ItemIndex := I;
        FunctionsLbClick(nil);
      end;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RemoveFunctionExecute(Sender: TObject);
var
  I: Integer;
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) then
  begin
    FSelectedKey.Functions.FreeAndDelete(FunctionsLb.ItemIndex);
    I := FunctionsLb.ItemIndex;
    RefreshFunctions;
    if I >= FunctionsLb.Items.Count then
      FunctionsLb.ItemIndex := FunctionsLb.Items.Count - 1
    else FunctionsLb.ItemIndex := I;
    FunctionsLbClick(nil);
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ModifyFunctionExecute(Sender: TObject);
var
  Func: PKeyFunction;
  I: Integer;
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) then
  begin
    if FFunctionForm = nil then
      FFunctionForm := TFunctionForm.Create(self);
    with FFunctionForm do
    begin
      RefreshVariables;
      RefreshBtnClick(nil);
      Func := FSelectedKey.Functions[FunctionsLb.ItemIndex];
      if (Func.Data.TargetTest = ttNoTest) and (Func.Data.VarName = '') then
      begin
        DefaultRb.Checked := True;
        DefaultRbClick(nil);
      end
      else begin
        TestRb.Checked := True;
        TestRbClick(nil);
        if Func.Data.TargetTest = ttNoTest then
          WindowCb.Checked := False
        else begin
          WindowCb.Checked := True;
          TargetTestCombo.ItemIndex := Integer(Func.Data.TargetTest) - 1;
          if Func.Data.TargetType = ttClass then
          begin
            ClassRb.Checked := True;
            ClassCombo.Text := Func.Data.Target;
          end
          else begin
            FormRb.Checked := True;
            FormCombo.Text := Func.Data.Target;
          end;
        end;
        WindowCbClick(nil);
        if Func.Data.VarName = '' then
          VariableCb.Checked := False
        else begin
          VariableCb.Checked := True;
          for I := 0 to Variables.Count - 1 do
            if Variables[I].Name = Func.Data.VarName then
            begin
              VarNameCombo.ItemIndex := I;
              VarNameComboChange(nil);
              VarValueCombo.ItemIndex := Func.Data.VarValue;
              Break;
            end;
        end;
        VariableCbClick(nil);
      end;
      if ShowModal = mrOk then
      begin
        with Func.Data^ do
        begin
          if TestRb.Checked and WindowCb.Checked then
          begin
            TargetTest := TTargetTest(TargetTestCombo.ItemIndex + 1);
            if ClassRb.Checked then
            begin
              Target := ClassCombo.Text;
              TargetType := ttClass;
            end
            else begin
              Target := FormCombo.Text;
              TargetType := ttForm;
            end;
          end
          else begin
            TargetTest := ttNoTest;
            Target := '';
            TargetType := ttNone;
          end;
          if TestRb.Checked and VariableCb.Checked then
          begin
            VarName := VarNameCombo.Items[VarNameCombo.ItemIndex];
            VarValue := VarValueCombo.ItemIndex;
          end
          else begin
            VarName := '';
            VarValue := -1;
          end;
        end;
        I := FunctionsLb.ItemIndex;
        RefreshFunctions;
        FunctionsLb.ItemIndex := I;
        FunctionsLbClick(nil);
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.AddActionExecute(Sender: TObject);
var
  I: Integer;
  Act: PSingleAction;
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) then
  begin
    if FActionForm = nil then
      FActionForm := TActionForm.Create(self);
    with FActionForm do
    begin
      RefreshBtnClick(nil);
      RefreshVariables;
      if ShowModal = mrOk then
      begin
        New(Act);
        FillChar(Act^, SizeOf(TSingleAction), 0);
        Act.KeybData := nil;
        Act.ExecData := nil;
        Act.AMixData := nil;
        Act.CVarData := nil;
        Act.MiscData := nil;
        case PageControl.ActivePageIndex of
          0: begin //emulate keyboard event
              Act.ActionType := atKeyb;
              New(Act.KeybData);
              FillChar(Act.KeybData^, SizeOf(TKeybData), 0);
              with Act.KeybData^ do
              begin
                Ch := KEYBKEYS[KeyCombo.ItemIndex].Ch;
                Alt := AltCb.Checked;
                Ctrl := CtrlCb.Checked;
                Shift := ShiftCb.Checked;
                Windows := WindowsCb.Checked;
                if KeybActiveWindowRb.Checked then
                begin TargetType := ttNone;
                  Target := '';
                end
                else if KeybClassRb.Checked then
                begin TargetType := ttClass;
                  Target := KeybClassCombo.Text;
                end
                else //if KeybFormRb.Checked then
                begin TargetType := ttForm;
                  Target := KeybFormCombo.Text;
                end;
              end;
            end;
          1: begin //execute program
              Act.ActionType := atExec;
              New(Act.ExecData);
              FillChar(Act.ExecData^, SizeOf(TExecData), 0);
              with Act.ExecData^ do
              begin
                Path := PathEdt.Text;
                FirstTarget := '';
                FirstClose := ttNone;
                FirstActivate := ttNone;
                if FirstTryCb.Checked then
                  if ExecClassRb.Checked then
                  begin
                    FirstTarget := ExecClassCombo.Text;
                    if CloseRb.Checked then FirstClose := ttClass
                    else FirstActivate := ttClass;
                  end
                  else begin
                    FirstTarget := ExecFormCombo.Text;
                    if CloseRb.Checked then FirstClose := ttForm
                    else FirstActivate := ttForm;
                  end;
              end;
            end;
          2: begin //volume control
              Act.ActionType := atAMix;
              New(Act.AMixData);
              FillChar(Act.AMixData^, SizeOf(TAMixData), 0);
              with Act.AMixData^ do
              begin
                Device := DeviceCombo.ItemIndex;
                Line := LineCombo.ItemIndex;
                Mute := MuteRb.Checked;
                Vol := IfThen(UpRb.Checked, VolTb.Position,
                  IfThen(DownRb.Checked, -VolTb.Position, 0));
              end;
            end;
          3: begin //change variable
              Act.ActionType := atCVar;
              New(Act.CVarData);
              FillChar(Act.CVarData^, SizeOf(TCVarData), 0);
              with Act.CVarData^ do
              begin
                VarName := VarNameCombo.Items[VarNameCombo.ItemIndex];
                if VarLoopUpRb.Checked then
                  Loop := LOOP_UP
                else if VarLoopDnRb.Checked then
                  Loop := LOOP_DOWN
                else Loop := NO_LOOP;
                SetValue := VarValueCombo.ItemIndex;
              end;
            end;
          4: begin //miscellaneous
              Act.ActionType := atMisc;
              New(Act.MiscData);
              FillChar(Act.MiscData^, SizeOf(TMiscData), 0);
              with Act.MiscData^ do
                if OSDRb.Checked then
                begin
                  Category := OSD_CAT;
                  Value := OSDCombo.ItemIndex;
                end
                else if MoreTVRb.Checked then
                begin
                  Category := MORETV_CAT;
                  Value := MoreTVCombo.ItemIndex;
                end
                else if KTVRb.Checked then
                begin
                  Category := KTV_CAT;
                  Value := KTVCombo.ItemIndex;
                end
                else if WinampRb.Checked then
                begin
                  Category := WINAMP_CAT;
                  Value := WinampCombo.ItemIndex;
                end
                else if MediaPlayerRb.Checked then
                begin
                  Category := MEDIAPLAYER_CAT;
                  Value := MediaPlayerCombo.ItemIndex;
                end
                else if BSPlayerRb.Checked then
                begin
                  Category := BSPLAYER_CAT;
                  Value := BSPlayerCombo.ItemIndex;
                end
                else if PowerDVDRb.Checked then
                begin
                  Category := POWERDVD_CAT;
                  Value := PowerDVDCombo.ItemIndex;
                end
                else if WinSTBRb.Checked then
                begin
                  Category := WINSTB_CAT;
                  Value := WinSTBCombo.ItemIndex;
                end
                {else if AnotherProgRb.Checked then
                begin
                  Category := ANOTHERPROG_CAT;
                  Value := AnotherProgCombo.ItemIndex;
                end};
            end;
        end; //case
        I := FSelectedKey.Functions[FunctionsLb.ItemIndex].Actions.Add(Act);
        RefreshActions;
        ActionsLb.ItemIndex := I;
        ActionsLbClick(nil);
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RemoveActionExecute(Sender: TObject);
var
  I: Integer;
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) and (ActionsLb.ItemIndex > -1) then
  begin
    FSelectedKey.Functions[FunctionsLb.ItemIndex].Actions.FreeAndDelete(ActionsLb.ItemIndex);
    I := ActionsLb.ItemIndex;
    RefreshActions;
    if I >= ActionsLb.Items.Count then
      ActionsLb.ItemIndex := ActionsLb.Items.Count - 1
    else ActionsLb.ItemIndex := I;
    ActionsLbClick(nil);
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ModifyActionExecute(Sender: TObject);
var
  I: Integer;
  Act: PSingleAction;
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) and (ActionsLb.ItemIndex > -1) then
  begin
    if FActionForm = nil then
      FActionForm := TActionForm.Create(self);
    with FActionForm do
    begin
      RefreshBtnClick(nil);
      RefreshVariables;
      Act := FSelectedKey.Functions[FunctionsLb.ItemIndex].Actions[ActionsLb.ItemIndex];
      case Act.ActionType of
        atKeyb: with Act.KeybData^ do
          begin
            PageControl.ActivePageIndex := 0;
            for I := 0 to NBKEYBKEYS - 1 do
              if KEYBKEYS[I].Ch = Ch then
              begin
                KeyCombo.ItemIndex := I;
                Break;
              end;
            AltCb.Checked := Alt;
            CtrlCb.Checked := Ctrl;
            ShiftCb.Checked := Shift;
            WindowsCb.Checked := Windows;
            case TargetType of
              ttNone: KeybActiveWindowRb.Checked := True;
              ttClass: begin KeybClassRb.Checked := True;
                  KeybClassCombo.Text := Target;
                end;
              ttForm: begin KeybFormRb.Checked := True;
                  KeybFormCombo.Text := Target;
                end;
            end;
          end;
        atExec: with Act.ExecData^ do
          begin
            PageControl.ActivePageIndex := 1;
            PathEdt.Text := Path;
            if (FirstTarget <> '') then
            begin
              FirstTryCb.Checked := True;
              FirstTryCbClick(nil);
              if (FirstClose <> ttNone) then
                CloseRb.Checked := True
              else ActivateRb.Checked := True;
              if (FirstClose = ttClass) or (FirstActivate = ttClass) then
              begin
                ExecClassRb.Checked := True;
                ExecClassCombo.Text := FirstTarget;
                ExecFormCombo.Text := '';
              end
              else begin
                ExecFormRb.Checked := True;
                ExecClassCombo.Text := '';
                ExecFormCombo.Text := FirstTarget;
              end;
            end;
          end;
        atAMix: with Act.AMixData^ do
          begin
            PageControl.ActivePageIndex := 2;
            DeviceCombo.ItemIndex := Device;
            LineCombo.ItemIndex := Line;
            if Mute then
              MuteRb.Checked := True
            else begin
              if Vol > 0 then
                UpRb.Checked := True
              else DownRb.Checked := True;
              VolTb.Position := Abs(Vol);
            end;
          end;
        atCVar: with Act.CVarData^ do
          begin
            PageControl.ActivePageIndex := 3;
            VarNameCombo.ItemIndex := VarNameCombo.Items.IndexOf(VarName);
            if Loop = LOOP_UP then
              VarLoopUpRb.Checked := True
            else if Loop = LOOP_DOWN then
              VarLoopDnRb.Checked := True
            else begin
              SetValueRb.Checked := True;
              VarValueCombo.ItemIndex := SetValue;
            end;
          end;
        atMisc: with Act.MiscData^ do
          begin
            PageControl.ActivePageIndex := 4;
            case Category of
              OSD_CAT: begin
                  OSDRb.Checked := True;
                  OSDCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(OSDCombo);
                end;
              MORETV_CAT: begin
                  MoreTVRb.Checked := True;
                  MoreTVCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(MoreTVCombo);
                end;
              KTV_CAT: begin
                  KTVRb.Checked := True;
                  KTVCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(KTVCombo);
                end;
              WINAMP_CAT: begin
                  WinampRb.Checked := True;
                  WinampCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(WinampCombo);
                end;
              MEDIAPLAYER_CAT: begin
                  MediaPlayerRb.Checked := True;
                  MediaPLayerCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(MediaPlayerCombo);
                end;
              BSPLAYER_CAT: begin
                  BSPlayerRb.Checked := True;
                  BSPLayerCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(BSPlayerCombo);
                end;
              POWERDVD_CAT: begin
                  PowerDVDRb.Checked := True;
                  PowerDVDCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(PowerDVDCombo);
                end;
              WINSTB_CAT: begin
                  WinSTBRb.Checked := True;
                  WinSTBCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(WinSTBCombo);
                end;
              {ANOTHERPROG_CAT: begin
                  AnotherProgRb.Checked := True;
                  AnotherProgCombo.ItemIndex := Value;
                  MiscSb.ScrollInView(AnotherProgCombo);
                end;}
            end;
          end;
      end;
      if ShowModal = mrOk then
      begin
        case Act.ActionType of
          atKeyb: Dispose(Act.KeybData);
          atExec: Dispose(Act.ExecData);
          atAMix: Dispose(Act.AMixData);
          atCVar: Dispose(Act.CVarData);
          atMisc: Dispose(Act.MiscData);
        end;
        case PageControl.ActivePageIndex of
          0: begin //emulate keyboard event
              Act.ActionType := atKeyb;
              New(Act.KeybData);
              FillChar(Act.KeybData^, SizeOf(TKeybData), 0);
              with Act.KeybData^ do
              begin
                Ch := KEYBKEYS[KeyCombo.ItemIndex].Ch;
                Alt := AltCb.Checked;
                Ctrl := CtrlCb.Checked;
                Shift := ShiftCb.Checked;
                Windows := WindowsCb.Checked;
                if KeybActiveWindowRb.Checked then
                begin TargetType := ttNone;
                  Target := '';
                end
                else if KeybClassRb.Checked then
                begin TargetType := ttClass;
                  Target := KeybClassCombo.Text;
                end
                else //if KeybFormRb.Checked then
                begin TargetType := ttForm;
                  Target := KeybFormCombo.Text;
                end;
              end;
            end;
          1: begin //execute program
              Act.ActionType := atExec;
              New(Act.ExecData);
              FillChar(Act.ExecData^, SizeOf(TExecData), 0);
              with Act.ExecData^ do
              begin
                Path := PathEdt.Text;
                FirstTarget := '';
                FirstClose := ttNone;
                FirstActivate := ttNone;
                if FirstTryCb.Checked then
                  if ExecClassRb.Checked then
                  begin
                    FirstTarget := ExecClassCombo.Text;
                    if CloseRb.Checked then FirstClose := ttClass
                    else FirstActivate := ttClass;
                  end
                  else begin
                    FirstTarget := ExecFormCombo.Text;
                    if CloseRb.Checked then FirstClose := ttForm
                    else FirstActivate := ttForm;
                  end;
              end;
            end;
          2: begin //volume control
              Act.ActionType := atAMix;
              New(Act.AMixData);
              FillChar(Act.AMixData^, SizeOf(TAMixData), 0);
              with Act.AMixData^ do
              begin
                Device := DeviceCombo.ItemIndex;
                Line := LineCombo.ItemIndex;
                Mute := MuteRb.Checked;
                Vol := IfThen(UpRb.Checked, VolTb.Position,
                  IfThen(DownRb.Checked, -VolTb.Position, 0));
              end;
            end;
          3: begin //change variable
              Act.ActionType := atCVar;
              New(Act.CVarData);
              FillChar(Act.CVarData^, SizeOf(TCVarData), 0);
              with Act.CVarData^ do
              begin
                VarName := VarNameCombo.Items[VarNameCombo.ItemIndex];
                if VarLoopUpRb.Checked then
                  Loop := LOOP_UP
                else if VarLoopDnRb.Checked then
                  Loop := LOOP_DOWN
                else Loop := NO_LOOP;
                SetValue := VarValueCombo.ItemIndex;
              end;
            end;
          4: begin //miscellaneous
              Act.ActionType := atMisc;
              New(Act.MiscData);
              FillChar(Act.MiscData^, SizeOf(TMiscData), 0);
              with Act.MiscData^ do
                if OSDRb.Checked then
                begin
                  Category := OSD_CAT;
                  Value := OSDCombo.ItemIndex;
                end
                else if MoreTVRb.Checked then
                begin
                  Category := MORETV_CAT;
                  Value := MoreTVCombo.ItemIndex;
                end
                else if KTVRb.Checked then
                begin
                  Category := KTV_CAT;
                  Value := KTVCombo.ItemIndex;
                end
                else if WinampRb.Checked then
                begin
                  Category := WINAMP_CAT;
                  Value := WinampCombo.ItemIndex;
                end
                else if MediaPlayerRb.Checked then
                begin
                  Category := MEDIAPLAYER_CAT;
                  Value := MediaPlayerCombo.ItemIndex;
                end
                else if BSPlayerRb.Checked then
                begin
                  Category := BSPLAYER_CAT;
                  Value := BSPlayerCombo.ItemIndex;
                end
                else if PowerDVDRb.Checked then
                begin
                  Category := POWERDVD_CAT;
                  Value := PowerDVDCombo.ItemIndex;
                end
                else if WinSTBRb.Checked then
                begin
                  Category := WINSTB_CAT;
                  Value := WinSTBCombo.ItemIndex;
                end
                {else if AnotherProgRb.Checked then
                begin
                  Category := ANOTHERPROG_CAT;
                  Value := AnotherProgCombo.ItemIndex;
                end};
            end;
        end; //case
        I := ActionsLb.ItemIndex;
        RefreshActions;
        ActionsLb.ItemIndex := I;
        ActionsLbClick(nil);
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.SaveExecute(Sender: TObject);
begin
  SaveIniFile;
  SaveKeyFile;
  ComPort.OnRxChar := MainForm.ComPortRxChar;
  Close;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.MoveFunctionUpExecute(Sender: TObject);
var
  OldItemIndex: Integer;
  FuncTmp: PKeyFunction;
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > 0) then
  begin
    OldItemIndex := FunctionsLb.ItemIndex;
    FuncTmp := FSelectedKey.Functions[OldItemIndex - 1];
    FSelectedKey.Functions[OldItemIndex - 1] := FSelectedKey.Functions[OldItemIndex];
    FSelectedKey.Functions[OldItemIndex] := FuncTmp;
    RefreshFunctions;
    FunctionsLb.ItemIndex := OldItemIndex - 1;
    FunctionsLbClick(nil);
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.MoveFunctionDownExecute(Sender: TObject);
var
  OldItemIndex: Integer;
  FuncTmp: PKeyFunction;
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) and (FunctionsLb.ItemIndex < FunctionsLb.Items.Count - 1) then
  begin
    OldItemIndex := FunctionsLb.ItemIndex;
    FuncTmp := FSelectedKey.Functions[OldItemIndex + 1];
    FSelectedKey.Functions[OldItemIndex + 1] := FSelectedKey.Functions[OldItemIndex];
    FSelectedKey.Functions[OldItemIndex] := FuncTmp;
    RefreshFunctions;
    FunctionsLb.ItemIndex := OldItemIndex + 1;
    FunctionsLbClick(nil);
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.MoveActionUpExecute(Sender: TObject);
begin
  //
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.MoveActionDownExecute(Sender: TObject);
begin
  //
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RemoteComboChange(Sender: TObject);
begin
  SetSelectedKey(nil);
  if RemoteCombo.ItemIndex = RemoteCombo.Items.Count - 1 then
  begin
    RemotePb.Visible := False;
    CustomKeyPnl.Visible := True;
    RefreshCustomKeys;
  end
  else with PRemote(FRemotes[RemoteCombo.ItemIndex])^ do
    begin
      CustomKeyPnl.Visible := False;
      RemotePb.Visible := True;
      if JPEG = nil then
      begin
        JPEG := TJPEGImage.Create;
        JPEG.LoadFromFile(JPEGStr);
      end;
      with RemotePb.Canvas do
      begin
        Brush.Color := clBtnFace;
        FillRect(Rect(0, 0, REMOTE_IMG_WIDTH, REMOTE_IMG_HEIGHT));
        Draw(0, 0, JPEG);
      end;
    end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.CustomKeysLbClick(Sender: TObject);
var
  I: Integer;
begin
  if CustomKeysLb.ItemIndex > -1 then
  begin
    I := CustomKeysLb.ItemIndex;
    SetSelectedKey(FCustomKeys[I]);
    if I >= CustomKeysLb.Items.Count then
      CustomKeysLb.ItemIndex := CustomKeysLb.Items.Count - 1
    else CustomKeysLb.ItemIndex := I;
    FunctionsLbClick(nil);
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.FunctionsLbClick(Sender: TObject);
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) then
  begin
    EnableOSDGb(True);
    EnableActionGb(True);
    OsdCombo.Text := FSelectedKey.Functions[FunctionsLb.ItemIndex].Data.OSD;
    RepeatActionsCb.Checked := FSelectedKey.Functions[FunctionsLb.ItemIndex].Data.RepeatActions;
    RefreshActions;
  end
  else begin
    EnableOSDGb(False);
    EnableActionGb(False);
    OsdCombo.Text := '';
    RepeatActionsCb.Checked := False;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.FunctionsLbDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  IndexStr: string;
  IndexStr2: string;
  ThePos: Integer;
begin
  with (Control as TListBox) do
  begin
    Canvas.FillRect(Rect);
    IconList.BkColor := Canvas.Brush.Color;
    IndexStr := Items[Index];
    if IndexStr = 'Default' then
    begin
      IconList.Draw(Canvas, Rect.Left, Rect.Top + 9, 2);
      Rect.Left := Rect.Left + 20;
      Canvas.TextRect(Rect, Rect.Left, Rect.Top + 11, IndexStr);
      Rect.Left := Rect.Left - 20;
    end
    else begin
      ThePos := Pos('#', IndexStr);
      if ThePos > 0 then
      begin
        IndexStr2 := Copy(IndexStr, ThePos + 1, Length(IndexStr) - ThePos);
        IndexStr := Copy(indexStr, 1, ThePos - 1);
      end
      else IndexStr2 := '';
      if IndexStr <> '' then
      begin
        if IndexStr2 = '' then
          Rect.Top := Rect.Top + 7;
        IconList.Draw(Canvas, Rect.Left, Rect.Top + 2, 0);
        Rect.Left := Rect.Left + 20;
        Canvas.TextRect(Rect, Rect.Left, Rect.Top + 4, IndexStr);
        Rect.Left := Rect.Left - 20;
      end;
      if IndexStr2 <> '' then
      begin
        if IndexStr = '' then
          Rect.Top := Rect.Top + 7
        else Rect.Top := Rect.Top + 16;
        IconList.Draw(Canvas, Rect.Left, Rect.Top + 2, 1);
        Rect.Left := Rect.Left + 20;
        Canvas.TextRect(Rect, Rect.Left, Rect.Top + 4, IndexStr2);
      end;
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.OsdComboChange(Sender: TObject);
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) then
    FSelectedKey.Functions[FunctionsLb.ItemIndex].Data.OSD := OsdCombo.Text;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.RepeatActionsCbClick(Sender: TObject);
begin
  if (FSelectedKey <> nil) and (FunctionsLb.ItemIndex > -1) then
    FSelectedKey.Functions[FunctionsLb.ItemIndex].Data.RepeatActions := RepeatActionsCb.Checked;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ActionsLbClick(Sender: TObject);
begin
  //
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ActionsLbDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  IndexStr: string;
  ImgIndex: Integer;
  Err: Integer;
begin
  with (Control as TListBox) do
  begin
    Canvas.FillRect(Rect);
    IconList.BkColor := Canvas.Brush.Color;
    IndexStr := Items[Index];
    if Length(IndexStr) > 1 then
    begin
      Val(IndexStr, ImgIndex, Err);
      Delete(IndexStr, 1, 2);
      IconList.Draw(Canvas, Rect.Left, Rect.Top + 9, ImgIndex);
      Rect.Left := Rect.Left + 20;
      Canvas.TextRect(Rect, Rect.Left, Rect.Top + 11, IndexStr);
    end
    else Canvas.TextRect(Rect, Rect.Left + 20, Rect.Top + 11, IndexStr);
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ShowComPortClick(Sender: TObject);
begin
  ComPortMemo.Visible := ShowComPort.Checked;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.ComPortRxChar(Sender: TObject; Count: Integer);
var
  I, Rd: Integer;
  Buf: array[0..1023] of Byte;
  KeyCode: string;

  //---------------------------------------------------------------
  function MatchCode(KeyCode1, KeyCode2: string): Boolean;
  var
    I, J: Integer;
    Equal: Boolean;
    ChEqual: Boolean;
    Key: PRemoteKey;
  begin
    Result := True;
    if (KeyCode1 = KeyCode2) then
      Exit;

    if (Pos('[', KeyCode1) > 0) or (Pos('_', KeyCode1) > 0) then
    begin
      New(Key);
      New(Key.Data);
      Key.Data.Code := KeyCode1;
      SetZarbCode(Key);
      if Key.NbZarbPoss > 1 then
      begin
        if Length(KeyCode2) = Length(Key.NewCode) div Key.NbZarbPoss then
        begin
          Equal := True;
          for I := 1 to Length(KeyCode2) do
          begin
            ChEqual := False;
            for J := 1 to Key.NbZarbPoss do
              if (KeyCode2[I] = Key.NewCode[Key.NbZarbPoss * (I - 1) + J])
              or (Key.NewCode[Key.NbZarbPoss * (I - 1) + J] = '_') then
              begin
                ChEqual := True;
                Break;
              end;
            if not ChEqual then
            begin
              Equal := False;
              Break;
            end;
          end;
          if Equal then
            Exit;
        end;
      end;
      Dispose(Key.Data);
      Dispose(Key);
    end;
    Result := False;
  end;

begin
  while ComPortMemo.Lines.Count > 10 do
    ComPortMemo.Lines.Delete(0);

  Rd := ComPort.Read(Buf, Count);
  KeyCode := '';
  for I := 0 to Rd - 1 do
    KeyCode := KeyCode + IntToHex(Buf[I], 2);
  if SpecialPCTV then
  begin
    I := Pos('FE', KeyCode);
    if I > 0 then
      KeyCode := Copy(KeyCode, I, 6)
    else KeyCode := '';
  end;
  ComPortMemo.Lines.Add(KeyCode);

  if (FSelectedKey = nil) or ((KeyCode <> FSelectedKey.Data.Code) and (KeyCode <> FSelectedKey.Data.RepeatCode)) then
    if RemoteCombo.ItemIndex < RemoteCombo.Items.Count - 1 then
      with PRemote(FRemotes[RemoteCombo.ItemIndex])^ do
        for I := 0 to Keys.Count - 1 do
        begin
          if MatchCode(PPhysicalKey(Keys[I]).Code, KeyCode) then
          begin
            with PPhysicalKey(Keys[I]).Rect do
              RemotePbMouseDown(nil, mbLeft, [], (Left + Right) div 2, (Top + Bottom) div 2);
            Break;
          end;
        end
    else
      for I := 0 to FCustomKeys.Count - 1 do
        if MatchCode(PRemoteKey(FCustomKeys[I]).Data.Code, KeyCode) then
        begin
          CustomKeysLb.ItemIndex := I;
          CustomKeysLbClick(nil);
        end;
end;

//------------------------------------------------------------------------------

procedure TConfigKeysForm.Label1Click(Sender: TObject);
begin
  ShellExecute(0, 'open', PChar(Label1.Caption), nil, nil, SW_SHOWNORMAL);
end;

end.

