unit UConfigOSD;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, IniFiles, UOSD, UGradPnl;

type
  TConfigOSDForm = class(TForm)
    BgColorBtn: TButton;
    TestPnl: TPanel;
    TestLbl: TLabel;
    Bevel1: TBevel;
    ShowTimeEdt: TLabeledEdit;
    ShowTimeUpDown: TUpDown;
    Bevel2: TBevel;
    VertPosLbl: TLabel;
    VertPosCombo: TComboBox;
    Bevel3: TBevel;
    OkBtn: TButton;
    HorizPosCombo: TComboBox;
    HorizPosLbl: TLabel;
    FontDialog: TFontDialog;
    ColorDialog: TColorDialog;
    TestBtn: TButton;
    FontBtn: TButton;
    AlphaBlendTb: TTrackBar;
    TransparentCb: TCheckBox;
    AlphaBlendCb: TCheckBox;
    BorderCb: TCheckBox;
    procedure BgColorBtnClick(Sender: TObject);
    procedure FontBtnClick(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
    procedure TestBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FGradPnl: TgradientPanel;
  public
    procedure ApplyLanguage(Lang: string);
  end;

var
  ConfigOSDForm: TConfigOSDForm;

implementation

uses Math, UMain;

{$R *.dfm}

//------------------------------------------------------------------------------

procedure TConfigOSDForm.ApplyLanguage(Lang: string);
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.lang')) do
  try
    FGradPnl.Caption := ReadString(Lang, 'OSDTitle', 'Configuration - OSD');
    FontBtn.Caption := ReadString(Lang, 'FontBtnCaption', 'Font...');
    BgColorBtn.Caption := ReadString(Lang, 'BgColorBtnCaption', 'Background color...');
    BorderCb.Caption := ReadString(Lang, 'BorderCbCaption', 'Border');
    TransparentCb.Caption := ReadString(Lang, 'TransparentCbCaption', 'Transparent');
    AlphaBlendCb.Caption := ReadString(Lang, 'AlphablendCbCaption', 'Alpha blend');
    ShowTimeEdt.EditLabel.Caption := ReadString(Lang, 'ShowTimeCaption', 'Show time (ms) :');
    VertPosLbl.Caption := ReadString(Lang, 'VerticalPosCaption', 'Vertical position :');
    HorizPosLbl.Caption := ReadString(Lang, 'HorizontalPosCaption', 'Horizontal position :');
    TestBtn.Caption := ReadString(Lang, 'TestBtnCaption', 'Test');
    OkBtn.Caption := ReadString(Lang, 'OkBtnCaption', 'Ok');
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigOSDForm.FormCreate(Sender: TObject);
var
  Str: string;
begin
  ClientHeight := 400;
  ClientWidth := 385;
  FGradPnl := TgradientPanel.Create(Self);
  with FGradPnl do
  begin
    Parent := Self;
    Align := alTop;
    Caption := 'Configuration - OSD';
    Font.Name := 'Arial';
    Font.Size := 14;
    Font.Style := [fsBold];
    Font.Color := clWhite;
    FEndColor := clBtnFace;;
    FBeginColor := clBlack;
  end;
  with TIniFile.Create(IniFileStr) do
  try
    FontDialog.Font.Name := ReadString('OSD', 'Font.Name', 'Courier New');
    FontDialog.Font.Color := StringToColor(ReadString('OSD', 'Font.Color', 'clLime'));
    FontDialog.Font.Size := ReadInteger('OSD', 'Font.Size', 48);
    if ReadBool('OSD', 'Font.Bold', False) then
      FontDialog.Font.Style := [fsBold]
    else FontDialog.Font.Style := [];
    if ReadBool('OSD', 'Font.Italic', False) then
      FontDialog.Font.Style := FontDialog.Font.Style + [fsItalic];
    if ReadBool('OSD', 'Font.Underline', False) then
      FontDialog.Font.Style := FontDialog.Font.Style + [fsUnderline];
    TestLbl.Font := FontDialog.Font;
    Str := ReadString('OSD', 'BgColor', 'clBlack');
    ColorDialog.Color := StringToColor(Str);
    TestLbl.Color := ColorDialog.Color;
    BorderCb.Checked := ReadBool('OSD', 'Border', True);
    TransparentCb.Checked := ReadBool('OSD', 'Transparent', False);
    AlphaBlendTb.Position := ReadInteger('OSD', 'AlphaBlend', 0);
    AlphaBlendCb.Checked := AlphaBlendTb.Position > 0;
    ShowTimeUpDown.Position := ReadInteger('OSD', 'ShowTime', 2500);
    VertPosCombo.ItemIndex := ReadInteger('OSD', 'VerticalPosition', 0);
    HorizPosCombo.ItemIndex := ReadInteger('OSD', 'HorizontalPosition', 0);
  finally
    Free;
  end;
  ApplyLanguage(CurrentLanguageStr);
end;

//------------------------------------------------------------------------------

procedure TConfigOSDForm.BgColorBtnClick(Sender: TObject);
begin
  if ColorDialog.Execute then
    TestLbl.Color := ColorDialog.Color;
end;

//------------------------------------------------------------------------------

procedure TConfigOSDForm.FontBtnClick(Sender: TObject);
begin
  if FontDialog.Execute then
    TestLbl.Font := FontDialog.Font;
end;

//------------------------------------------------------------------------------

procedure TConfigOSDForm.TestBtnClick(Sender: TObject);
begin
  with OSDForm do
  begin
    SetAlignement(TVertAlign(VertPosCombo.ItemIndex), THorizAlign(HorizPosCombo.ItemIndex));
    MessageLbl.Font.Name := FontDialog.Font.Name;
    MessageLbl.Font.Color := FontDialog.Font.Color;
    MessageLbl.Font.Size := FontDialog.Font.Size;
    MessageLbl.Font.Style := FontDialog.Font.Style;
    MessageLbl.Color := ColorDialog.Color;
    Color := MessageLbl.Font.Color;
    TimerInterval := ShowTimeUpDown.Position;
    BorderWidth := IfThen(BorderCb.Checked, 3, 0);
    TransparentColorValue := ColorDialog.Color;
    TransparentColor := TransparentCb.Checked;
    AlphaBlend := AlphaBlendCb.Checked;
    AlphaBlendValue := AlphaBlendTb.Position;
    if Sender = TestBtn then
      ShowMessage('Test OSD', '')
    else
      ShowMessage('OSD configured', '')
  end;
end;

//------------------------------------------------------------------------------

procedure TConfigOSDForm.OkBtnClick(Sender: TObject);
begin
  with TIniFile.Create(IniFileStr) do
  try
    EraseSection('OSD');
    WriteString('OSD', 'Font.Name', FontDialog.Font.Name);
    WriteString('OSD', 'Font.Color', ColorToString(FontDialog.Font.Color));
    WriteInteger('OSD', 'Font.Size', FontDialog.Font.Size);
    WriteBool('OSD', 'Font.Bold', fsBold in FontDialog.Font.Style);
    WriteBool('OSD', 'Font.Italic', fsItalic in FontDialog.Font.Style);
    WriteBool('OSD', 'Font.Underline', fsUnderline in FontDialog.Font.Style);
    WriteString('OSD', 'BgColor', ColorToString(ColorDialog.Color));
    WriteBool('OSD', 'Border', BorderCb.Checked);
    WriteBool('OSD', 'Transparent', TransparentCb.Checked);
    WriteInteger('OSD', 'AlphaBlend', ifThen(AlphaBlendCb.Checked, AlphaBlendTb.Position, 0));
    WriteInteger('OSD', 'ShowTime', ShowTimeUpDown.Position);
    WriteInteger('OSD', 'VerticalPosition', VertPosCombo.ItemIndex);
    WriteInteger('OSD', 'HorizontalPosition', HorizPosCombo.ItemIndex);
  finally
    Free;
  end;
  TestBtnClick(nil);
  Close;
end;

//------------------------------------------------------------------------------

end.

