unit UFunction;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Math, IniFiles, UMain, UVarDef;

type
  TFunctionForm = class(TForm)
    DefaultRb: TRadioButton;
    TestRb: TRadioButton;
    TestGb: TGroupBox;
    WindowCb: TCheckBox;
    TargetTestCombo: TComboBox;
    FormCombo: TComboBox;
    RefreshBtn: TButton;
    VariableCb: TCheckBox;
    VariablesBtn: TButton;
    VarNameCombo: TComboBox;
    VarValueCombo: TComboBox;
    EqualLbl: TLabel;
    OkBtn: TButton;
    CancelBtn: TButton;
    ClassCombo: TComboBox;
    Bevel1: TBevel;
    ClassRb: TRadioButton;
    FormRb: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure DefaultRbClick(Sender: TObject);
    procedure TestRbClick(Sender: TObject);
    procedure WindowCbClick(Sender: TObject);
    procedure VariableCbClick(Sender: TObject);
    procedure VarNameComboChange(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure VariablesBtnClick(Sender: TObject);
    procedure ComboChange(Sender: TObject);
  private
    { Private declarations }
  public
    procedure RefreshVariables;
    procedure ApplyLanguage(Lang: string);
  end;

var
  FunctionForm: TFunctionForm;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------

procedure TFunctionForm.ApplyLanguage(Lang: string);
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.lang')) do
  try
    Self.Caption := ReadString(Lang, 'FunctionTitle', 'Function');
    DefaultRb.Caption := ReadString(Lang, 'DefaultCaption', 'Default');
    TestRb.Caption := ReadString(Lang, 'MakeTestCaption', 'Make a test :');
    TestGb.Caption := ReadString(Lang, 'TestCaption', 'Test');
    WindowCb.Caption := ReadString(Lang, 'TestWindowCaption', 'Test window');
    RefreshBtn.Caption := ReadString(Lang, 'RefreshBtnCaption', 'Refresh');
    TargetTestCombo.Items[0] := ReadString(Lang, 'ActiveWindowIsText', 'Active window is');
    TargetTestCombo.Items[1] := ReadString(Lang, 'WindowExists', 'Window exists');
    ClassRb.Caption := ReadString(Lang, 'TestClassCaption', 'Test class :');
    FormRb.Caption := ReadString(Lang, 'TestFormCaption', 'Test form :');
    VariableCb.Caption := ReadString(Lang, 'TestVariableCaption', 'Test variable');
    VariablesBtn.Caption := ReadString(Lang, 'DefineVarBtnCaption', 'Define variables');
    OkBtn.Caption := ReadString(Lang, 'OkBtnCaption', 'Ok');
    CancelBtn.Caption := ReadString(Lang, 'CancelBtnCaption', 'Cancel');
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.FormCreate(Sender: TObject);
begin
  ClientHeight := 300;
  ClientWidth := 480;
  TargetTestCombo.ItemIndex := 0;
  RefreshBtnClick(nil);
  ApplyLanguage(CurrentLanguageStr);
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.DefaultRbClick(Sender: TObject);
begin
  WindowCb.Enabled := False;
  VariableCb.Enabled := False;
  WindowCbClick(nil);
  VariableCbClick(nil);
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.TestRbClick(Sender: TObject);
begin
  WindowCb.Enabled := True;
  VariableCb.Enabled := True;
  WindowCbClick(nil);
  VariableCbClick(nil);
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.WindowCbClick(Sender: TObject);
begin
  TargetTestCombo.Enabled := WindowCb.Checked and WindowCb.Enabled;
  ClassCombo.Enabled := WindowCb.Checked and WindowCb.Enabled;
  FormCombo.Enabled := WindowCb.Checked and WindowCb.Enabled;
  RefreshBtn.Enabled := WindowCb.Checked and WindowCb.Enabled;
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.VariableCbClick(Sender: TObject);
begin
  VarNameCombo.Enabled := VariableCb.Checked and VariableCb.Enabled;
  VarValueCombo.Enabled := VariableCb.Checked and VariableCb.Enabled;
  EqualLbl.Enabled := VariableCb.Checked and VariableCb.Enabled;
  VariablesBtn.Enabled := VariableCb.Checked and VariableCb.Enabled;
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.VarNameComboChange(Sender: TObject);
begin
  VarValueCombo.Clear;
  if VarNameCombo.ItemIndex > -1 then
    VarValueCombo.Items.AddStrings(Variables[VarNameCombo.ItemIndex].Values);
  if VarValueCombo.Items.Count > 0 then
    VarValueCombo.ItemIndex := 0;
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.RefreshBtnClick(Sender: TObject);
var
  AHandle: HWnd;
  PCharResult: PChar;
  TextLength: Integer;
begin
  ClassCombo.Items.Clear;
  FormCombo.Items.Clear;
  AHandle := GetWindow(Handle, GW_HWNDFIRST);
  while Ahandle <> NULL_HANDLE do
  begin
    TextLength := GetWindowTextLength(AHandle) + 1;
    if TextLength > 1 then
    begin
      GetMem(PCharResult, 255);
      GetClassName(AHandle, PCharResult, 255);
      ClassCombo.Items.Add(string(PCharResult));
      Freemem(PCharResult);
      GetMem(PCharResult, TextLength);
      GetWindowText(AHandle, PCharResult, TextLength);
      FormCombo.Items.Add(string(PCharResult));
      Freemem(PCharResult);
    end;
    AHandle := GetNextWindow(AHandle, GW_HWNDNEXT);
  end;
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.VariablesBtnClick(Sender: TObject);
begin
  with TVarDefForm.Create(Self) do
  try
    ShowModal;
    RefreshVariables;
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.RefreshVariables;
var
  I: Integer;
  OldVarName: string;
  OldVarValue: Integer;
begin
  OldVarName := VarNameCombo.Items[VarNameCombo.ItemIndex];
  OldVarValue := Max(VarValueCombo.ItemIndex, 0);
  VarNameCombo.Clear;
  VarValueCombo.Clear;
  if Variables.Count > 0 then
  begin
    for I := 0 to Variables.Count - 1 do
      VarNameCombo.Items.Add(Variables[I].Name);
    VarNameCombo.ItemIndex := 0;
    VarNameComboChange(nil);
  end;
  if VarNameCombo.Items.IndexOf(OldVarName) > -1 then
    VarNameCombo.ItemIndex := VarNameCombo.Items.IndexOf(OldVarName);
  if OldVarValue < VarValueCombo.Items.Count then
    VarValueCombo.ItemIndex := OldVarValue;
end;

//------------------------------------------------------------------------------

procedure TFunctionForm.ComboChange(Sender: TObject);
begin
  ClassRb.Checked := Sender = ClassCombo;
  FormRb.Checked := Sender = FormCombo;
end;

end.

