unit UMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ImgList, Menus, ActnList, Contnrs, IniFiles, USystray, UAMixer,
  UComPort, StdCtrls, StrUtils, ExtCtrls;

type
  TVertAlign = (valTop, valCenter, valBottom);
  THorizAlign = (halLeft, halCenter, halRight);
  TTargetType = (ttNone, ttClass, ttForm);
  TTargetTest = (ttNoTest, ttActiveWindowIs, ttWindowOpen);
  TActionType = (atKeyb, atExec, atAMix, atCVar, atMisc);

  POldKeybData = ^TOldKeybData;
  TOldKeybData = record
    Ch: Char;
    Alt: Boolean;
    Ctrl: Boolean;
    Shift: Boolean;
    Windows: Boolean;
  end;

  PKeybData = ^TKeybData;
  TKeybData = record
    Target: string[64];
    TargetType: TTargetType;
    Ch: Char;
    Alt: Boolean;
    Ctrl: Boolean;
    Shift: Boolean;
    Windows: Boolean;
  end;

  PExecData = ^TExecData;
  TExecData = record
    Path: string[255];
    FirstTarget: string[64];
    FirstClose: TTargetType;
    FirstActivate: TTargetType;
  end;

  PAMixData = ^TAMixData;
  TAMixData = record
    Device: Integer;
    Line: Integer;
    Vol: Integer;
    Mute: Boolean;
  end;

  PCVarData = ^TCVarData;
  TCVarData = record
    VarName: string[32];
    Loop: Byte;
    SetValue: Integer;
  end;

  PMiscData = ^TMiscData;
  TMiscData = record
    Category: Integer;
    Value: Integer;
  end;

  PSingleAction = ^TSingleAction;
  TSingleAction = record
    ActionType: TActionType;
    KeybData: PKeybData;
    ExecData: PExecData;
    AMixData: PAMixData;
    CVarData: PCVarData;
    MiscData: PMiscData;
  end;

  TSingleActionList = class(TList)
  private
    function GetAction(Index: Integer): PSingleAction;
    procedure SetAction(Index: Integer; Value: PSingleAction);
  public
    procedure FreeAndDelete(Index: Integer);
    property Actions[Index: Integer]: PSingleAction read GetAction write SetAction; default;
  end;

  PFuncData = ^TFuncData;
  TFuncData = record
    TargetTest: TTargetTest;
    Target: string[64];
    TargetType: TTargetType;
    VarName: string[32];
    VarValue: Integer;
    OSD: string[64];
    RepeatActions: Boolean;
  end;

  PKeyFunction = ^TKeyFunction;
  TKeyFunction = record
    Data: PFuncData;
    Actions: TSingleActionList;
  end;

  TFunctionList = class(TList)
  private
    function GetFunction(Index: Integer): PKeyFunction;
    procedure SetFunction(Index: Integer; Value: PKeyFunction);
  public
    procedure FreeAndDelete(Index: Integer);
    property Functions[Index: Integer]: PKeyFunction read GetFunction write SetFunction; default;
  end;

  PRKeyData = ^TRKeyData;
  TRKeyData = record
    Name: string[64];
    Code: string[32];
    RepeatCode: string[32];
    GenericRepeatCode: Boolean;
  end;

  PRemoteKey = ^TRemoteKey;
  TRemoteKey = record
    Data: PRKeyData;
    Remote: Integer;
    NbZarbPoss: Integer;
    NewCode: string;
    Functions: TFunctionList;
  end;

  TKeyList = class(TList)
  private
    function GetKey(Index: Integer): PRemoteKey;
    procedure SetKey(Index: Integer; Value: PRemoteKey);
  public
    procedure FreeAndDelete(Index: Integer);
    property Keys[Index: Integer]: PRemoteKey read GetKey write SetKey; default;
  end;

  PVariable = ^TVariable;
  TVariable = record
    Name: string;
    Value: Integer;
    Values: TStringList;
  end;

  TVariableList = class(TList)
  private
    function GetVar(Index: Integer): PVariable;
    procedure SetVar(Index: Integer; Value: PVariable);
  public
    function GetVarFromName(Name: string): PVariable;
    property Vars[Index: Integer]: PVariable read GetVar write SetVar; default;
  end;

  TMainForm = class(TForm)
    SystrayPopup: TPopupMenu;
    Configurekeys: TMenuItem;
    ConfigureCOMport: TMenuItem;
    ConfigureOSD: TMenuItem;
    N1: TMenuItem;
    Startminimized1: TMenuItem;
    N2: TMenuItem;
    ClosePCRemote: TMenuItem;
    ActionList: TActionList;
    ConfigKeysAction: TAction;
    ConfigCOMAction: TAction;
    ConfigOSDAction: TAction;
    StartMiniAction: TAction;
    CloseAction: TAction;
    DebugOutput: TMemo;
    RemoteOffImg: TImage;
    RemotePbImg: TImage;
    RemoteOnImg: TImage;
    Configurelanguage1: TMenuItem;
    ConfigLangAction: TAction;
    procedure FormCreate(Sender: TObject);
    procedure ConfigKeysActionExecute(Sender: TObject);
    procedure ConfigCOMActionExecute(Sender: TObject);
    procedure ConfigOSDActionExecute(Sender: TObject);
    procedure ConfigLangActionExecute(Sender: TObject);
    procedure CloseActionExecute(Sender: TObject);
    procedure StartMiniActionExecute(Sender: TObject);
    procedure ComPortRxChar(Sender: TObject; Count: Integer);
  private
    { Private declarations }
  public
    procedure ApplyLanguage(Lang: string);
  end;

var
  MainForm: TMainForm;
  SpecialPCTV: Boolean;
  IniFileStr: string;
  KeyFileStr: string;
  CurrentLanguageStr: string;
  TrayIcon: TTrayIcon;
  AMixer: TAudioMixer;
  ComPort: TComPort;
  Keys: TKeyList;
  Variables: TVariableList;
  PreviousKey: string;

const
  ANIMATE_INTERVAL = 100;
  NULL_HANDLE = 0;
  OSD_CAT = 0;
  MORETV_CAT = 1;
  KTV_CAT = 2;
  WINAMP_CAT = 3;
  MEDIAPLAYER_CAT = 4;
  BSPLAYER_CAT = 5;
  POWERDVD_CAT = 6;
  WINSTB_CAT = 7;
  //ANOTHERPROG_CAT = 8;
  NO_LOOP = 0;
  LOOP_UP = 1;
  LOOP_DOWN = 2;
  crHand = 1;

procedure GetDirList(Path: string; var List: TStringList);
function COMConnect(_Port: string; _BaudRate, _DataBits, _StopBits, _ParityBits, _FlowControl: Integer): Boolean;
procedure InitSystray(Title: string; Popup: TPopupMenu; MainIcon: TIcon);
procedure FinalSysTray;
procedure SaveIniFile;
procedure LoadIniFile;
procedure SaveKeyFile;
procedure LoadKeyFile;
procedure WizardExecute;
procedure ShowOSD(Msg: string);
function SearchTarget(Target: string; TargetType: TTargetType): HWND;
procedure SetZarbCode(Key: PRemoteKey);

implementation

uses UConfigKeys, UConfigCOMPort, UConfigOSD, UConfigLang, UOSD, UAction, UWizard;

{$R *.dfm}

//------------------------------------------------------------------------------

procedure GetDirList(Path: string; var List: TStringList);
var
  Sr: TSearchRec;

  function IsDirectory(Sr: TSearchRec): Boolean;
  begin
    Result := (Sr.Name <> '.') and (Sr.Name <> '..') and (Sr.Attr and faDirectory > 0);
  end;

begin
  if List <> nil then
  begin
    List.Clear;
    if (FindFirst(Path, faDirectory, Sr) = 0) then
    begin
      if IsDirectory(Sr) then
        List.Add(Sr.Name);
      while (FindNext(Sr) = 0) do
        if IsDirectory(Sr) then
          List.Add(Sr.Name);
      FindClose(Sr);
      List.Sort;
    end;
  end;
end;

//------------------------------------------------------------------------------

function COMConnect(_Port: string; _BaudRate, _DataBits, _StopBits, _ParityBits, _FlowControl: Integer): Boolean;
begin
  if ComPort = nil then
    ComPort := TComPort.Create(MainForm)
  else ComPort.Close;
  with ComPort do
  try
    Port := _Port;
    BaudRate := TBaudRate(_BaudRate);
    DataBits := TDataBits(_DataBits);
    StopBits := TStopBits(_StopBits);
    Parity.Bits := TParityBits(_ParityBits);
    FlowControl.FlowControl := TFlowControl(_FlowControl);
    OnRxChar := MainForm.ComPortRxChar;
    Open;
    Result := ComPort.Connected;
  except
    Result := False;
  end;
end;

//------------------------------------------------------------------------------

procedure InitSystray(Title: string; Popup: TPopupMenu; MainIcon: TIcon);
begin
  if TrayIcon = nil then
    TrayIcon := TTrayIcon.Create(nil);
  with TrayIcon do
  begin
    Icon := MainIcon;
    PopupMenu := Popup;
    Tooltip := Title;
    OnDblClick := MainForm.ConfigKeysActionExecute;
    Active := True;
  end;
end;

//------------------------------------------------------------------------------

procedure FinalSysTray;
begin
  if TrayIcon <> nil then
  begin
    TrayIcon.Active := False;
    TrayIcon.Free;
  end;
end;

//------------------------------------------------------------------------------

procedure SaveIniFile;
var
  I, J: Integer;
  SectionStr: string;
begin
  with TIniFile.Create(IniFileStr) do
  try
    //delete all variables in the inifile
    I := 0;
    while SectionExists('Variable' + IntToStr(I)) do
    begin
      EraseSection('Variable' + IntToStr(I));
      Inc(I);
    end;
    //write all variables in the inifile
    for I := 0 to Variables.Count - 1 do
      with Variables[I]^ do
      begin
        SectionStr := 'Variable' + IntToStr(I);
        WriteString(SectionStr, 'Name', Name);
        for J := 0 to Values.Count - 1 do
          WriteString(SectionStr, 'Value' + IntToStr(J), Values[J]);
      end;
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure LoadIniFile;
var
  I, J: Integer;
  SectionStr: string;
  Str: string;
  Variable: PVariable;
begin
  with TIniFile.Create(IniFileStr) do
  try
    CurrentLanguageStr := ReadString('Main', 'Language', 'English');
    //load COM
    if COMConnect(ReadString('ComPort', 'Name', 'COM1'),
      ReadInteger('ComPort', 'BaudRate', 4),
      ReadInteger('ComPort', 'DataBits', 3),
      ReadInteger('ComPort', 'StopBits', 0),
      ReadInteger('ComPort', 'ParityBits', 0),
      ReadInteger('ComPort', 'FlowControl', 0)) then
      InitSystray('PCRemote', MainForm.SystrayPopup, MainForm.RemoteOffImg.Picture.Icon)
    else
      InitSystray('PCRemote', MainForm.SystrayPopup, MainForm.RemotePbImg.Picture.Icon);
    //load Main
    SpecialPCTV := ReadBool('Main', 'SpecialPCTV', False);
    MainForm.StartMiniAction.Checked := ReadBool('Main', 'LoadMinimized', True);
    //load Variables
    I := 0;
    SectionStr := 'Variable0';
    while SectionExists(SectionStr) do
    begin
      New(Variable);
      Variable.Name := ReadString(SectionStr, 'Name', '');
      Variable.Value := 0;
      Variable.Values := TStringList.Create;
      J := 0;
      Str := ReadString(SectionStr, 'Value0', '');
      while Str <> '' do
      begin
        Variable.Values.Add(Str);
        Inc(J);
        Str := ReadString(SectionStr, 'Value' + IntToStr(J), '');
      end;
      Variables.Add(Variable);
      Inc(I);
      SectionStr := 'Variable' + IntToStr(I);
    end;
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure SaveKeyFile;
var
  I, J, K, L: Integer;
  F: file;
begin
  //delete keys that have no function
  I := 0;
  while I < Keys.Count do
  begin
    while (I < Keys.Count) and (Keys[I].Functions.Count = 0) do
      Keys.Delete(I);
    Inc(I);
  end;
  //open file
  AssignFile(F, KeyFileStr);
  Rewrite(F, 1);
  //write all keys
  I := Keys.Count;
  BlockWrite(F, I, SizeOf(Integer));
  for I := 0 to Keys.Count - 1 do
  begin
    BlockWrite(F, Keys[I].Data^, SizeOf(TRKeyData));
    //write all functions
    J := Keys[I].Functions.Count;
    BlockWrite(F, J, SizeOf(Integer));
    for J := 0 to Keys[I].Functions.Count - 1 do
    begin
      BlockWrite(F, Keys[I].Functions[J].Data^, SizeOf(TFuncData));
      //write all actions
      K := Keys[I].Functions[J].Actions.Count;
      BlockWrite(F, K, SizeOf(Integer));
      for K := 0 to Keys[I].Functions[J].Actions.Count - 1 do
        with Keys[I].Functions[J].Actions[K]^ do
        begin
          L := Integer(ActionType);
          BlockWrite(F, L, SizeOf(Integer));
          case ActionType of
            atKeyb: BlockWrite(F, KeybData^, SizeOf(TKeybData));
            atExec: BlockWrite(F, ExecData^, SizeOf(TExecData));
            atAMix: BlockWrite(F, AMixData^, SizeOf(TAMixData));
            atCVar: BlockWrite(F, CVarData^, SizeOf(TCVarData));
            atMisc: BlockWrite(F, MiscData^, SizeOf(TMiscData));
          end;
        end;
    end;
  end;
  //close file
  CloseFile(F);
end;

//------------------------------------------------------------------------------

procedure LoadKeyFile;
var
  Key: PRemoteKey;
  Func: PKeyFunction;
  Act: PSingleAction;
  ICount, JCount, KCount: Integer;
  I, J, K, L: Integer;
  F: file;
begin
  if not FileExists(KeyFileStr) then
    WizardExecute;
  if FileExists(KeyFileStr) then
  begin
    //open file
    AssignFile(F, KeyFileStr);
    FileMode := fmOpenRead;
    Reset(F, 1);
    //read all keys
    BlockRead(F, ICount, SizeOf(Integer));
    for I := 0 to ICount - 1 do
    begin
      New(Key);
      New(Key.Data);
      Key.Remote := -1;
      Key.Functions := TFunctionList.Create;
      BlockRead(F, Key.Data^, SizeOf(TRKeyData));
      SetZarbCode(Key);
      //read all functions
      BlockRead(F, JCount, SizeOf(Integer));
      for J := 0 to JCount - 1 do
      begin
        New(Func);
        New(Func.Data);
        Func.Actions := TSingleActionList.Create;
        BlockRead(F, Func.Data^, SizeOf(TFuncData));
        //read all actions
        BlockRead(F, KCount, SizeOf(Integer));
        for K := 0 to KCount - 1 do
        begin
          New(Act);
          BlockRead(F, L, SizeOf(Integer));
          Act.ActionType := TActionType(L);
          case Act.ActionType of
            atKeyb: begin
                New(Act.KeybData);
                BlockRead(F, Act.KeybData^, SizeOf(TKeybData));
              end;
            atExec: begin
                New(Act.ExecData);
                BlockRead(F, Act.ExecData^, SizeOf(TExecData));
              end;
            atAMix: begin
                New(Act.AMixData);
                BlockRead(F, Act.AMixData^, SizeOf(TAMixData));
              end;
            atCVar: begin
                New(Act.CVarData);
                BlockRead(F, Act.CVarData^, SizeOf(TCVarData));
              end;
            atMisc: begin
                New(Act.MiscData);
                BlockRead(F, Act.MiscData^, SizeOf(TMiscData));
              end;
          end;
          Func.Actions.Add(Act);
        end;
        Key.Functions.Add(Func);
      end;
      Keys.Add(Key);
    end;
    //close file
    CloseFile(F);
  end;
end;

//------------------------------------------------------------------------------

procedure WizardExecute;
var
  I: Integer;
  Rem: PRemote;

  function AddDefFunction(ARemote: PRemote; AKeyIndex: Integer; AActionType: TActionType; ARep: Boolean; AOSD: string = ''): PSingleAction;
  var
    I: Integer;
    AKey: PRemoteKey;
    AFunc: PKeyFunction;
    AAct: PSingleAction;
  begin
    AKey := nil;
    for I := 0 to Keys.Count - 1 do
      if Keys[I].Data.Code = PPhysicalKey(ARemote.Keys[AKeyIndex]).Code then
        AKey := Keys[I];
    if AKey = nil then
    begin
      New(AKey);
      FillChar(AKey^, SizeOf(TRemoteKey), 0);
      New(AKey.Data);
      FillChar(AKey.Data^, SizeOf(TRKeyData), 0);
      AKey.Data.Name := PPhysicalKey(ARemote.Keys[AKeyIndex]).Name;
      AKey.Data.Code := PPhysicalKey(ARemote.Keys[AKeyIndex]).Code;
      AKey.Data.RepeatCode := PPhysicalKey(ARemote.Keys[AKeyIndex]).RepeatCode;
      AKey.Data.GenericRepeatCode := PPhysicalKey(ARemote.Keys[AKeyIndex]).GenericRepeatCode;
      AKey.Remote := 0;
      AKey.Functions := TFunctionList.Create;
      Keys.Add(AKey);
    end;
    New(AFunc);
    FillChar(AFunc^, SizeOf(TKeyFunction), 0);
    New(AFunc.Data);
    FillChar(AFunc.Data^, SizeOf(TFuncData), 0);
    with AFunc.Data^ do
    begin
      TargetTest := ttNoTest;
      Target := '';
      TargetType := ttNone;
      VarName := '';
      VarValue := -1;
      OSD := AOSD;
      RepeatActions := ARep;
    end;
    AFunc.Actions := TSingleActionList.Create;
    AKey.Functions.Add(AFunc);
    New(AAct);
    FillChar(AAct^, SizeOf(TSingleAction), 0);
    AAct.KeybData := nil;
    AAct.ExecData := nil;
    AAct.AMixData := nil;
    AAct.CVarData := nil;
    AAct.MiscData := nil;
    AAct.ActionType := AActionType;
    case AActionType of
      atKeyb: begin
          New(AAct.KeybData);
          FillChar(AAct.KeybData^, SizeOf(TKeybData), 0);
          AAct.KeybData.Target := '';
          AAct.KeybData.TargetType := ttNone;
          AAct.KeybData.Ch := #0;
          AAct.KeybData.Alt := False;
          AAct.KeybData.Ctrl := False;
          AAct.KeybData.Shift := False;
          AAct.KeybData.Windows := False;
        end;
      atExec: begin
          New(AAct.ExecData);
          FillChar(AAct.ExecData^, SizeOf(TExecData), 0);
          AAct.ExecData.Path := '';
          AAct.ExecData.FirstTarget := '';
          AAct.ExecData.FirstClose := ttNone;
          AAct.ExecData.FirstActivate := ttNone;
        end;
      atAMix: begin
          New(AAct.AMixData);
          FillChar(AAct.AMixData^, SizeOf(TAMixData), 0);
          AAct.AMixData.Device := 0;
          AAct.AMixData.Line := 0;
          AAct.AMixData.Vol := 0;
          AAct.AMixData.Mute := False;
        end;
    end;
    {AAct.ActionType := atMisc;
    New(AAct.MiscData);
    FillChar(AAct.MiscData^, SizeOf(TMiscData), 0);
    with AAct.MiscData^ do
    begin
      Category := ACategory;
      Value := AValue;
    end;}
    AFunc.Actions.Add(AAct);
    Result := AAct;
  end;

  procedure AddFunction(ARemote: PRemote; AKeyIndex: Integer;
    ATargetTest: TTargetTest; ATarget: string; ATargetType: TTargetType;
    ACategory, AValue: Integer; AOSD: string; ARepeatActions: Boolean);
  var
    I: Integer;
    AKey: PRemoteKey;
    AFunc: PKeyFunction;
    AAct: PSingleAction;
  begin
    AKey := nil;
    for I := 0 to Keys.Count - 1 do
      if Keys[I].Data.Code = PPhysicalKey(ARemote.Keys[AKeyIndex]).Code then
        AKey := Keys[I];
    if AKey = nil then
    begin
      New(AKey);
      FillChar(AKey^, SizeOf(TRemoteKey), 0);
      New(AKey.Data);
      FillChar(AKey.Data^, SizeOf(TRKeyData), 0);
      AKey.Data.Name := PPhysicalKey(ARemote.Keys[AKeyIndex]).Name;
      AKey.Data.Code := PPhysicalKey(ARemote.Keys[AKeyIndex]).Code;
      AKey.Data.RepeatCode := PPhysicalKey(ARemote.Keys[AKeyIndex]).RepeatCode;
      AKey.Data.GenericRepeatCode := PPhysicalKey(ARemote.Keys[AKeyIndex]).GenericRepeatCode;
      AKey.Remote := 0;
      AKey.Functions := TFunctionList.Create;
      Keys.Add(AKey);
    end;
    New(AFunc);
    FillChar(AFunc^, SizeOf(TKeyFunction), 0);
    New(AFunc.Data);
    FillChar(AFunc.Data^, SizeOf(TFuncData), 0);
    with AFunc.Data^ do
    begin
      TargetTest := ATargetTest;
      Target := ATarget;
      TargetType := ATargetType;
      VarName := '';
      VarValue := -1;
      OSD := AOSD;
      RepeatActions := ARepeatActions;
    end;
    AFunc.Actions := TSingleActionList.Create;
    AKey.Functions.Add(AFunc);
    New(AAct);
    FillChar(AAct^, SizeOf(TSingleAction), 0);
    AAct.KeybData := nil;
    AAct.ExecData := nil;
    AAct.AMixData := nil;
    AAct.CVarData := nil;
    AAct.MiscData := nil;
    AAct.ActionType := atMisc;
    New(AAct.MiscData);
    FillChar(AAct.MiscData^, SizeOf(TMiscData), 0);
    with AAct.MiscData^ do
    begin
      Category := ACategory;
      Value := AValue;
    end;
    AFunc.Actions.Add(AAct);
  end;

begin
  with TWizardForm.Create(MainForm) do
  try
    if ShowModal = mrOk then
    begin
      ComPort.Close;
      ComPort.Port := COMCombo.Items[COMCombo.ItemIndex];
      ComPort.Open;
      with TIniFile.Create(IniFileStr) do
      try
        WriteString('ComPort', 'Name', ComPort.Port);
        WriteString('Main', 'Language', CurrentLanguageStr);
      finally
        Free;
      end;
      CurrentLanguageStr := LanguageCombo.Items[LanguageCombo.ItemIndex];
      ConfigKeysForm := TConfigKeysForm.Create(MainForm);
      Rem := nil;
      for I := 0 to ConfigKeysForm.Remotes.Count - 1 do
        with PRemote(ConfigKeysForm.Remotes[I])^ do
          if (Name = 'Pinnacle Studio PCTV') and (Keys.Count = 41) then
            Rem := ConfigKeysForm.Remotes[I];
      if Rem <> nil then
      begin
        for I := 0 to SoftsLb.Items.Count - 1 do
          if SoftsLb.Checked[I] then
            if SoftsLb.Items[I] = MoreTVStr then
            begin
              AddFunction(Rem, 0, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 15, '', False);
              AddFunction(Rem, 1, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 16, '', False);
              AddFunction(Rem, 2, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 17, '', False);
              AddFunction(Rem, 4, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 18, '', False);
              AddFunction(Rem, 5, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 19, '', False);
              AddFunction(Rem, 6, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 20, '', False);
              AddFunction(Rem, 8, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 21, '', False);
              AddFunction(Rem, 9, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 22, '', False);
              AddFunction(Rem, 10, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 23, '', False);
              AddFunction(Rem, 13, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 24, '', False);
              AddFunction(Rem, 12, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 9, '', False);
              AddFunction(Rem, 22, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 25, '', False);
              AddFunction(Rem, 26, ttWindowOpen, 'MoreTV', ttClass, MORETV_CAT, 26, '', False);
            end
            else if SoftsLb.Items[I] = KTVStr then
            begin
              AddFunction(Rem, 0, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 1, '', False);
              AddFunction(Rem, 1, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 2, '', False);
              AddFunction(Rem, 2, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 3, '', False);
              AddFunction(Rem, 4, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 4, '', False);
              AddFunction(Rem, 5, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 5, '', False);
              AddFunction(Rem, 6, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 6, '', False);
              AddFunction(Rem, 8, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 7, '', False);
              AddFunction(Rem, 9, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 8, '', False);
              AddFunction(Rem, 10, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 9, '', False);
              AddFunction(Rem, 13, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 0, '', False);
              AddFunction(Rem, 22, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 14, '', True);
              AddFunction(Rem, 26, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 15, '', True);
              AddFunction(Rem, 12, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 24, '', False);
              AddFunction(Rem, 25, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 26, '', True);
              AddFunction(Rem, 23, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 27, '', True);
              AddFunction(Rem, 24, ttWindowOpen, 'K!TV XP', ttClass, KTV_CAT, 35, '', False);
            end
            else if SoftsLb.Items[I] = Winamp2Str then
            begin
              AddFunction(Rem, 29, ttWindowOpen, 'Winamp v1.x', ttClass, WINAMP_CAT, 0, 'Prev track', False);
              AddFunction(Rem, 31, ttWindowOpen, 'Winamp v1.x', ttClass, WINAMP_CAT, 2, 'Pause', False);
              AddFunction(Rem, 32, ttWindowOpen, 'Winamp v1.x', ttClass, WINAMP_CAT, 4, 'Next track', False);
              AddFunction(Rem, 34, ttWindowOpen, 'Winamp v1.x', ttClass, WINAMP_CAT, 1, 'Play', False);
              AddFunction(Rem, 35, ttWindowOpen, 'Winamp v1.x', ttClass, WINAMP_CAT, 3, 'Stop', False);
              AddFunction(Rem, 33, ttWindowOpen, 'Winamp v1.x', ttClass, WINAMP_CAT, 5, '', True);
              AddFunction(Rem, 36, ttWindowOpen, 'Winamp v1.x', ttClass, WINAMP_CAT, 6, '', True);
            end
            else if SoftsLb.Items[I] = Winamp3Str then
            begin
              AddFunction(Rem, 29, ttWindowOpen, 'STUDIO', ttClass, WINAMP_CAT, 0, 'Prev track', False);
              AddFunction(Rem, 31, ttWindowOpen, 'STUDIO', ttClass, WINAMP_CAT, 2, 'Pause', False);
              AddFunction(Rem, 32, ttWindowOpen, 'STUDIO', ttClass, WINAMP_CAT, 4, 'Next track', False);
              AddFunction(Rem, 34, ttWindowOpen, 'STUDIO', ttClass, WINAMP_CAT, 1, 'Play', False);
              AddFunction(Rem, 35, ttWindowOpen, 'STUDIO', ttClass, WINAMP_CAT, 3, 'Stop', False);
              AddFunction(Rem, 33, ttWindowOpen, 'STUDIO', ttClass, WINAMP_CAT, 5, '', True);
              AddFunction(Rem, 36, ttWindowOpen, 'STUDIO', ttClass, WINAMP_CAT, 6, '', True);
            end
            else if SoftsLb.Items[I] = Winamp5Str then
            begin
              AddFunction(Rem, 29, ttWindowOpen, 'BaseWindow_RootWnd', ttClass, WINAMP_CAT, 0, 'Prev track', False);
              AddFunction(Rem, 31, ttWindowOpen, 'BaseWindow_RootWnd', ttClass, WINAMP_CAT, 2, 'Pause', False);
              AddFunction(Rem, 32, ttWindowOpen, 'BaseWindow_RootWnd', ttClass, WINAMP_CAT, 4, 'Next track', False);
              AddFunction(Rem, 34, ttWindowOpen, 'BaseWindow_RootWnd', ttClass, WINAMP_CAT, 1, 'Play', False);
              AddFunction(Rem, 35, ttWindowOpen, 'BaseWindow_RootWnd', ttClass, WINAMP_CAT, 3, 'Stop', False);
              AddFunction(Rem, 33, ttWindowOpen, 'BaseWindow_RootWnd', ttClass, WINAMP_CAT, 5, '', True);
              AddFunction(Rem, 36, ttWindowOpen, 'BaseWindow_RootWnd', ttClass, WINAMP_CAT, 6, '', True);
            end
            else if SoftsLb.Items[I] = WMPStr then
            begin
              AddFunction(Rem, 12, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 11, '', False);
              AddFunction(Rem, 29, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 2, '', False);
              AddFunction(Rem, 31, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 0, '', False);
              AddFunction(Rem, 32, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 3, '', False);
              AddFunction(Rem, 33, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 4, '', True);
              AddFunction(Rem, 34, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 0, '', False);
              AddFunction(Rem, 35, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 1, '', False);
              AddFunction(Rem, 36, ttWindowOpen, 'WMPlayerApp', ttClass, MEDIAPLAYER_CAT, 5, '', True);
            end
            else if SoftsLb.Items[I] = BSPlayerStr then
            begin
              AddFunction(Rem, 12, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 8, '', False);
              AddFunction(Rem, 14, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 9, '', False);
              AddFunction(Rem, 19, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 29, '', True);
              AddFunction(Rem, 21, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 30, '', True);
              AddFunction(Rem, 22, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 0, '', True);
              AddFunction(Rem, 23, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 2, '', True);
              AddFunction(Rem, 24, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 14, '', False);
              AddFunction(Rem, 25, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 3, '', True);
              AddFunction(Rem, 26, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 1, '', True);
              AddFunction(Rem, 28, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 38, '', False);
              AddFunction(Rem, 29, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 12, '', False);
              AddFunction(Rem, 31, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 14, '', False);
              AddFunction(Rem, 32, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 16, '', False);
              AddFunction(Rem, 33, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 17, '', False);
              AddFunction(Rem, 34, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 13, '', False);
              AddFunction(Rem, 35, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 15, '', False);
              AddFunction(Rem, 36, ttWindowOpen, 'BSPlayer', ttClass, BSPLAYER_CAT, 18, '', False);
            end
            else if SoftsLb.Items[I] = PowerDVDStr then
            begin
              AddFunction(Rem, 12, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 20, '', False);
              AddFunction(Rem, 19, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 17, '', False);
              AddFunction(Rem, 21, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 18, '', False);
              AddFunction(Rem, 22, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 7, '', False);
              AddFunction(Rem, 23, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 5, '', False);
              AddFunction(Rem, 24, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 1, '', False);
              AddFunction(Rem, 25, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 6, '', False);
              AddFunction(Rem, 26, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 9, '', False);
              AddFunction(Rem, 27, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 19, '', False);
              AddFunction(Rem, 29, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 3, '', False);
              AddFunction(Rem, 31, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 1, '', False);
              AddFunction(Rem, 32, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 4, '', False);
              AddFunction(Rem, 33, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 5, '', True);
              AddFunction(Rem, 34, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 0, '', False);
              AddFunction(Rem, 35, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 2, '', False);
              AddFunction(Rem, 36, ttWindowOpen, 'PowerDVD', ttForm, POWERDVD_CAT, 6, '', True);
            end
            else if SoftsLb.Items[I] = WinSTBStr then
            begin
              AddFunction(Rem, 0, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 0, '', False);
              AddFunction(Rem, 1, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 1, '', False);
              AddFunction(Rem, 2, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 2, '', False);
              AddFunction(Rem, 4, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 3, '', False);
              AddFunction(Rem, 5, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 4, '', False);
              AddFunction(Rem, 6, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 5, '', False);
              AddFunction(Rem, 8, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 6, '', False);
              AddFunction(Rem, 9, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 7, '', False);
              AddFunction(Rem, 10, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 8, '', False);
              AddFunction(Rem, 13, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 9, '', False);
              AddFunction(Rem, 19, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 24, '', False);
              AddFunction(Rem, 21, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 25, '', False);
              AddFunction(Rem, 16, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 22, '', True);
              AddFunction(Rem, 20, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 23, '', True);
              AddFunction(Rem, 22, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 17, '', True);
              AddFunction(Rem, 23, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 19, '', True);
              AddFunction(Rem, 24, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 21, '', False);
              AddFunction(Rem, 25, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 20, '', True);
              AddFunction(Rem, 26, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 18, '', True);
              AddFunction(Rem, 27, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 16, '', False);
              AddFunction(Rem, 28, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 14, '', False);
              AddFunction(Rem, 29, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 15, '', False);
              AddFunction(Rem, 11, ttWindowOpen, 'TSoftOSDForm', ttClass, WINSTB_CAT, 26, '', False);
            end
            {else if SoftsLb.Items[I] = AnotherProgStr then
            begin
            end};
        if OffCb.Checked then
          with AddDefFunction(Rem, 3, atKeyb, False).KeybData^ do
          begin
            Ch := #115;
            Alt := True;
          end;
        if PCb.Checked then
          with AddDefFunction(Rem, 7, atKeyb, False).KeybData^ do
          begin
            Ch := #9;
            Alt := True;
            Shift := True;
          end;
        if VolPlusCb.Checked then
          AddDefFunction(Rem, 16, atAMix, True, 'Vol [vol_percent]%').AMixData.Vol := 2048;
        if MuteAllCb.Checked then
          AddDefFunction(Rem, 17, atAMix, False, 'Mute').AMixData.Mute := True;
        if PageUpCb.Checked then
          AddDefFunction(Rem, 19, atKeyb, False).KeybData.Ch := #33;
        if VolMinusCb.Checked then
          AddDefFunction(Rem, 20, atAMix, True, 'Vol [vol_percent]%').AMixData.Vol := -2048;
        if PageDnCb.Checked then
          AddDefFunction(Rem, 21, atKeyb, False).KeybData^.Ch := #34;
        if UpCb.Checked then
          AddDefFunction(Rem, 22, atKeyb, True).KeybData^.Ch := #38;
        if LeftCb.Checked then
          AddDefFunction(Rem, 23, atKeyb, True).KeybData^.Ch := #37;
        if EnterCb.Checked then
          AddDefFunction(Rem, 24, atKeyb, False).KeybData^.Ch := #13;
        if RightCb.Checked then
          AddDefFunction(Rem, 25, atKeyb, True).KeybData^.Ch := #39;
        if DownCb.Checked then
          AddDefFunction(Rem, 26, atKeyb, True).KeybData^.Ch := #40;
        if MenuCb.Checked then
          with AddDefFunction(Rem, 27, atKeyb, False).KeybData^ do
          begin
            Ch := #27;
            Ctrl := True;
          end;
        if TabCb.Checked then
          AddDefFunction(Rem, 28, atKeyb, False).KeybData^.Ch := #9;
        if FileExists(TVEdt.Text) then
          AddDefFunction(Rem, 37, atExec, False, ChangeFileExt(ExtractFileName(TVEdt.Text), '')).ExecData.Path := TVEdt.Text;
        if FileExists(TeletextEdt.Text) then
          AddDefFunction(Rem, 38, atExec, False, ChangeFileExt(ExtractFileName(TeletextEdt.Text), '')).ExecData.Path := TeletextEdt.Text;
        if FileExists(RadioEdt.Text) then
          AddDefFunction(Rem, 39, atExec, False, ChangeFileExt(ExtractFileName(RadioEdt.Text), '')).ExecData.Path := RadioEdt.Text;
        if FileExists(EPGEdt.Text) then
          AddDefFunction(Rem, 40, atExec, False, ChangeFileExt(ExtractFileName(EPGEdt.Text), '')).ExecData.Path := EPGEdt.Text;
      end;
    end;
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure ShowOSD(Msg: string);
var
  I: Integer;
  Msg1, Msg2: string;

  //-----------------------------------------------------------
  function GetPercent(MixerDevice, MixerLine: Integer): string;
  var
    L, R, M: Integer;
    VD, MD: Boolean;
    Stereo: Boolean;
  begin
    AMixer.GetVolume(MixerDevice, MixerLine, L, R, M, Stereo, VD, MD);
    Result := IntToStr(100 * L div 65535);
  end;

  //---------------------------------------------------------
  function GetGauge(MixerDevice, MixerLine: Integer): string;
  var
    Value: Integer;
    L, R, M: Integer;
    VD, MD: Boolean;
    Stereo: Boolean;
  begin
    AMixer.GetVolume(MixerDevice, MixerLine, L, R, M, Stereo, VD, MD);
    Value := 16 * L div 65535;
    Result := StringOfChar('|', Value) + StringOfChar('-', 16 - Value);
  end;

begin
  if Msg = '#Clock#' then
    if OSDForm.ClockMode then
    begin
      OSDForm.ClockMode := False;
      OSDForm.TimerTimer(nil);
    end
    else begin
      OSDForm.ClockMode := True;
      OSDForm.TimerTimer(nil);
    end
  else begin
    if Pos('[', Msg) > 0 then
    begin
      if Pos('[vol_percent]', Msg) > 0 then
      begin
        Msg1 := StuffString(Msg, Pos('[vol_percent]', Msg), 13, GetPercent(0, -1));
        Msg2 := '';
      end
      else if Pos('[vol_gauge]', Msg) > 0 then
      begin
        Msg1 := LeftStr(Msg, Pos('[', Msg) - 1);
        Msg2 := GetGauge(0, -1);
      end
      else if Pos('_value]', Msg) > 0 then
      begin
        for I := 0 to Variables.Count - 1 do
          if Pos('[' + Variables[I].Name + '_value]', Msg) > 0 then
          begin
            Msg1 := StuffString(Msg, Pos('[' + Variables[I].Name + '_value]', Msg), Length(Variables[I].Name) + 8, Variables[I].Values[Variables[I].Value]);
            Msg2 := '';
          end;
      end
      else if AMixer.Destinations.Count > 0 then
      begin
        for I := 0 to AMixer.Destinations[0].Connections.Count - 1 do
          if Pos('[' + AMixer.Destinations[0].Connections[I].Data.szShortName + '_gauge]', Msg) > 0 then
          begin
            Msg1 := LeftStr(Msg, Pos('[', Msg) - 1);
            Msg2 := GetGauge(0, I);
          end
          else if Pos('[' + AMixer.Destinations[0].Connections[I].Data.szShortName + '_percent]', Msg) > 0 then
          begin
            Msg1 := StuffString(Msg, Pos('[', Msg), Pos(']', Msg) - Pos('[', Msg) + 1, GetPercent(0, I));
            Msg2 := '';
          end
      end
      else begin
        Msg1 := '';
        Msg2 := '';
      end;
    end
    else begin
      Msg1 := Msg;
      Msg2 := '';
    end;
    OSDForm.ShowMessage(Msg1, Msg2);
  end;
end;

//------------------------------------------------------------------------------

function SearchTarget(Target: string; TargetType: TTargetType): HWND;
var
  Str: string;
  AHandle: HWND;
  PCharResult: PChar;
  TextLength: Integer;
begin
  Result := NULL_HANDLE;
  if Target <> '' then
  begin
    AHandle := GetWindow(MainForm.Handle, GW_HWNDFIRST);
    while Ahandle <> NULL_HANDLE do
    begin
      case TargetType of
        ttForm: begin
            TextLength := GetWindowTextLength(AHandle) + 1;
            if TextLength > 1 then
            begin
              GetMem(PCharResult, TextLength);
              GetWindowText(AHandle, PCharResult, TextLength);
              Str := string(PCharResult);
              Freemem(PCharResult);
            end;
          end;
        ttClass: begin
            GetMem(PCharResult, 255);
            GetClassName(AHandle, PCharResult, 255);
            Str := string(PCharResult);
            Freemem(PCharResult);
          end;
      end;
      if SameText(Target, Str) then
      begin
        Result := AHandle;
        Exit;
      end;
      AHandle := GetNextWindow(AHandle, GW_HWNDNEXT);
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure SetZarbCode(Key: PRemoteKey);
var
  I, J, K: Integer;
  InBrackets: Boolean;
  CodeLength, Count: Integer;

  //-------------------------------
  procedure SizeCount(Str: string);
  var
    I, J: Integer;
  begin
    CodeLength := 0;
    Count := 1;
    J := 0;
    InBrackets := False;
    for I := 1 to Length(Str) do
      case Str[I] of
        '[': begin
            InBrackets := True;
            J := 1;
          end;
        '|': Inc(J);
        ']': begin
            InBrackets := False;
            if J > Count then
              Count := J;
            Inc(CodeLength);
          end;
        else if not InBrackets then Inc(CodeLength);
      end;
  end;

begin
{  if  Key.Data.GenericRepeatCode and (Length(Key.Data.RepeatCode) < Length(Key.Data.Code)) then
    Key.Data.RepeatCode := Key.Data.Code + Key.Data.RepeatCode;}
  if (Pos('[', Key.Data.Code) > 0) or (Pos('_', Key.Data.Code) > 0) then
  begin
    SizeCount(Key.Data.Code);
    Key.NbZarbPoss := Count;
    Key.NewCode := StringOfChar('.', CodeLength * Count);
    J := 1;
    K := 0;
    InBrackets := False;
    for I := 1 to Length(Key.Data.Code) do
      case Key.Data.Code[I] of
        '[': begin
            InBrackets := True;
          end;
        '|': Inc(J);
        ']': begin
            InBrackets := False;
            J := 1;
            Inc(K);
          end;
        else begin
            Key.NewCode[K * Count + J] := Key.Data.Code[I];
            if not InBrackets then
              Inc(K);
          end;
      end;
  end
  else begin
    Key.NbZarbPoss := 1;
    Key.NewCode := Key.Data.Code;
  end;
end;

//------------------------------------------------------------------------------

function IsForegroundWindow(Target: string; TargetType: TTargetType): Boolean;
var
  AHandle: HWND;
  PCharResult: PChar;
  TextLength: Integer;
begin
  Result := False;
  if Target <> '' then
  begin
    AHandle := GetForegroundWindow;
    case TargetType of
      ttForm: begin
          TextLength := GetWindowTextLength(AHandle) + 1;
          if TextLength > 1 then
          begin
            GetMem(PCharResult, TextLength);
            GetWindowText(AHandle, PCharResult, TextLength);
            Result := string(PCharResult) = Target;
            Freemem(PCharResult);
          end;
        end;
      ttClass: begin
          GetMem(PCharResult, 255);
          GetClassName(AHandle, PCharResult, 255);
          Result := string(PCharResult) = Target;
          Freemem(PCharResult);
        end;
    end;
  end;
end;

//------------------------------------------------------------------------------

function TSingleActionList.GetAction(Index: Integer): PSingleAction;
begin
  Result := PSingleAction(Items[Index]);
end;

//------------------------------------------------------------------------------

procedure TSingleActionList.SetAction(Index: Integer; Value: PSingleAction);
begin
  Items[Index] := Value;
end;

//------------------------------------------------------------------------------

procedure TSingleActionList.FreeAndDelete(Index: Integer);
begin
  case Actions[Index].ActionType of
    atKeyb: Dispose(Actions[Index].KeybData);
    atExec: Dispose(Actions[Index].ExecData);
    atAMix: Dispose(Actions[Index].AMixData);
    atCVar: Dispose(Actions[Index].CVarData);
    atMisc: Dispose(Actions[Index].MiscData);
  end;
  Dispose(Actions[Index]);
  Delete(Index);
end;

//------------------------------------------------------------------------------

function TFunctionList.GetFunction(Index: Integer): PKeyFunction;
begin
  Result := PKeyFunction(Items[Index]);
end;

//------------------------------------------------------------------------------

procedure TFunctionList.SetFunction(Index: Integer; Value: PKeyFunction);
begin
  Items[Index] := Value;
end;

//------------------------------------------------------------------------------

procedure TFunctionList.FreeAndDelete(Index: Integer);
begin
  while Functions[Index].Actions.Count > 0 do
    Functions[Index].Actions.FreeAndDelete(0);
  Functions[Index].Actions.Free;
  Dispose(Functions[Index].Data);
  Dispose(Functions[Index]);
  Delete(Index);
end;

//------------------------------------------------------------------------------

function TKeyList.GetKey(Index: Integer): PRemoteKey;
begin
  Result := PRemoteKey(Items[Index]);
end;

//------------------------------------------------------------------------------

procedure TKeyList.SetKey(Index: Integer; Value: PRemoteKey);
begin
  Items[Index] := Value;
end;

//------------------------------------------------------------------------------

procedure TKeyList.FreeAndDelete(Index: Integer);
begin
  while Keys[Index].Functions.Count > 0 do
    Keys[Index].Functions.FreeAndDelete(0);
  Keys[Index].Functions.Free;
  Dispose(Keys[Index].Data);
  Dispose(Keys[Index]);
  Delete(Index);
end;

//------------------------------------------------------------------------------

function TVariableList.GetVar(Index: Integer): PVariable;
begin
  Result := PVariable(Items[Index]);
end;

//------------------------------------------------------------------------------

procedure TVariableList.SetVar(Index: Integer; Value: PVariable);
begin
  Items[Index] := Value;
end;

//------------------------------------------------------------------------------

function TVariableList.GetVarFromName(Name: string): PVariable;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    if GetVar(I).Name = Name then
    begin
      Result := GetVar(I);
      Exit;
    end;
  Result := nil;
end;

//------------------------------------------------------------------------------

procedure TMainForm.ApplyLanguage(Lang: string);
var
  I: Integer;
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.lang')) do
  try
    ConfigKeysAction.Caption := ReadString(Lang, 'ConfigKeysCaption', 'Configure keys...');
    ConfigCOMAction.Caption := ReadString(Lang, 'ConfigCOMCaption', 'Configure COM port...');
    ConfigOSDAction.Caption := ReadString(Lang, 'ConfigOSDCaption', 'Configure OSD...');
    ConfigLangAction.Caption := ReadString(Lang, 'ConfigLangCaption', 'Configure language...');
    StartMiniAction.Caption := ReadString(Lang, 'StartMinimizedCaption', 'Start minimized');
    CloseAction.Caption := ReadString(Lang, 'CloseCaption', 'Close PCRemote');
    for I := 0 to NBMORETVOPT - 1 do
      MORETVOPT[I] := ReadString(Lang, 'MoreTv' + IntToStr(I), 'MoreTv' + IntToStr(I));
    for I := 0 to NBKTVOPT - 1 do
      KTVOPT[I] := ReadString(Lang, 'KTV' + IntToStr(I), 'KTV' + IntToStr(I));
    for I := 0 to NBWINAMPOPT - 1 do
      WINAMPOPT[I] := ReadString(Lang, 'Winamp' + IntToStr(I), 'Winamp' + IntToStr(I));
    for I := 0 to NBWMPOPT - 1 do
      WMPOPT[I] := ReadString(Lang, 'WMP' + IntToStr(I), 'WMP' + IntToStr(I));
    for I := 0 to NBBSPLAYOPT - 1 do
      BSPLAYOPT[I] := ReadString(Lang, 'BsPlayer' + IntToStr(I), 'BsPlayer' + IntToStr(I));
    for I := 0 to NBPDVDOPT - 1 do
      PDVDOPT[I] := ReadString(Lang, 'PowerDVD' + IntToStr(I), 'PowerDVD' + IntToStr(I));
    for I := 0 to NBWINSTBOPT - 1 do
      WINSTBOPT[I] := ReadString(Lang, 'WinSTB' + IntToStr(I), 'WinSTB' + IntToStr(I));
    {for I := 0 to NBANOTHERPROGOPT - 1 do
      ANOTHERPROGOPT[I] := ReadString(Lang, 'AnotherProg' + IntToStr(I), 'AnotherProg' + IntToStr(I));}
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Screen.Cursors[crHand] := LoadCursor(0, IDC_HAND);
  IniFileStr := ExtractFilePath(ParamStr(0)) + 'PCRemote.ini';
  KeyFileStr := ExtractFilePath(ParamStr(0)) + 'Keys.data';
  Application.ShowMainForm := False;
  Keys := TKeyList.Create;
  Variables := TVariableList.Create;
  AMixer := TAudioMixer.Create;
  LoadIniFile;
  ApplyLanguage(CurrentLanguageStr);
  LoadKeyFile;
  TrayIcon.ChangeIcon(RemoteOnImg.Picture.Icon, ANIMATE_INTERVAL, RemoteOffImg.Picture.Icon);
end;

//------------------------------------------------------------------------------

procedure TMainForm.ConfigKeysActionExecute(Sender: TObject);
begin
  if ConfigKeysForm <> nil then
  begin
    ComPort.OnRxChar := ConfigKeysForm.ComPortRxChar;
    ConfigKeysForm.Show;
  end
  else ConfigKeysForm := TConfigKeysForm.Create(Self);
end;

//------------------------------------------------------------------------------

procedure TMainForm.ConfigCOMActionExecute(Sender: TObject);
begin
  if ConfigCOMPortForm <> nil then
    ConfigCOMPortForm.Show
  else ConfigCOMPortForm := TConfigCOMPortForm.Create(Self);
end;

//------------------------------------------------------------------------------

procedure TMainForm.ConfigOSDActionExecute(Sender: TObject);
begin
  if ConfigOSDForm <> nil then
    ConfigOSDForm.Show
  else ConfigOSDForm := TConfigOSDForm.Create(Self);
end;

//------------------------------------------------------------------------------

procedure TMainForm.ConfigLangActionExecute(Sender: TObject);
begin
  if ConfigLangForm <> nil then
    ConfigLangForm.Show
  else ConfigLangForm := TConfigLangForm.Create(Self);
end;

//------------------------------------------------------------------------------

procedure TMainForm.StartMiniActionExecute(Sender: TObject);
begin
  StartMiniAction.Checked := not (StartMiniAction.Checked);
  with TIniFile.Create(IniFileStr) do
  try
    WriteBool('Main', 'LoadMinimized', MainForm.StartMiniAction.Checked);
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TMainForm.CloseActionExecute(Sender: TObject);
begin
  FinalSysTray;
  Close;
end;

//------------------------------------------------------------------------------

procedure ReadKey(Count: Integer; var Key: PRemoteKey; var Repeating: Boolean);
var
  I: Integer;
  Rd: Integer;
  Buf: array[0..1023] of Byte;
  KeyCode: string;
  KeepPreviousCode: Boolean;
  NewPreviousCodeAssigned: Boolean;

  //------------------------------------------------------------
  function MatchCode(Key: PRemoteKey; KeyCode: string): Boolean;
  var
    I, J: Integer;
    Equal: Boolean;
    ChEqual: Boolean;
  begin
    Result := True;
    if (Key.Data.Code = KeyCode) then
      Exit;
    if Key.Data.GenericRepeatCode then
    begin
      if Key.Data.RepeatCode = PreviousKey + KeyCode then
      begin
        KeepPreviousCode := True;
        Repeating := True;
        Exit;
      end
    end
    else if Key.Data.RepeatCode = KeyCode then
    begin
      Repeating := True;
      Exit;
    end;

    if Key.NbZarbPoss > 1 then
    begin
      if Length(KeyCode) = Length(Key.NewCode) div Key.NbZarbPoss then
      begin
        Equal := True;
        for I := 1 to Length(KeyCode) do
        begin
          ChEqual := False;
          for J := 1 to Key.NbZarbPoss do
            if (KeyCode[I] = Key.NewCode[Key.NbZarbPoss * (I - 1) + J])
            or (Key.NewCode[Key.NbZarbPoss * (I - 1) + J] = '_') then
            begin
              ChEqual := True;
              Break;
            end;
          if not ChEqual then
          begin
            Equal := False;
            Break;
          end;
        end;
        if Equal then
          Exit;
      end;
    end;

    Result := False;
  end;

begin
  Key := nil;
  Repeating := False;
  Rd := ComPort.Read(Buf, Count);
  KeyCode := '';
  for I := 0 to Rd - 1 do
    KeyCode := KeyCode + IntToHex(Buf[I], 2);
  if SpecialPCTV then
  begin
    I := Pos('FE', KeyCode);
    if I > 0 then
      KeyCode := Copy(KeyCode, I, 6)
    else KeyCode := '';
  end;

  KeepPreviousCode := False;
  NewPreviousCodeAssigned := False;
  if (KeyCode <> '') and (KeyCode <> '00') then
    for I := 0 to Keys.Count - 1 do
      if MatchCode(Keys[I], KeyCode) then
      begin
        if not KeepPreviousCode then
        begin
          PreviousKey := Keys[I].Data.Code;
          NewPreviousCodeAssigned := True;
        end;
        Key := Keys[I];
      end;
  if not KeepPreviousCode and not NewPreviousCodeAssigned then
    PreviousKey := KeyCode;
end;

//------------------------------------------------------------------------------

procedure TMainForm.ComPortRxChar(Sender: TObject; Count: Integer);
var
  Key: PRemoteKey;
  Repeating: Boolean;
  I, J, K: Integer;
  Res: Boolean;
  TargetStr: string;
begin
  ReadKey(Count, Key, Repeating);
  if Key <> nil then
    //search function
    for J := 0 to Key.Functions.Count - 1 do
      with Key.Functions[J]^ do
      begin
        if (Data.TargetTest = ttNoTest) and (Data.VarName = '') then
          Res := True
        else begin
          //close OSD window (eventually)
          {if GetForegroundWindow = OpaqueOSDForm.Handle then
          begin
            OpaqueOSDForm.Timer.Enabled := False;
            ShowWindow(OpaqueOSDForm.handle, SW_HIDE);
          end;}
          //test window
          if (Data.TargetTest <> ttNoTest) then
            case Data.TargetTest of
              ttActiveWindowIs:
                Res := IsForegroundWindow(Data.Target, Data.TargetType);
              ttWindowOpen: begin
//                Res := SearchTarget(Data.Target, Data.TargetType) <> NULL_HANDLE;
                  TargetStr := Data.Target;
                  case Data.TargetType of
                    ttClass: Res := FindWindow(PChar(TargetStr), nil) <> NULL_HANDLE;
                    ttForm: Res := FindWindow(nil, PChar(TargetStr)) <> NULL_HANDLE;
                    else Res := False;
                  end;
                end;
              else Res := False;
            end
          else Res := False;
            //test variable
          if Res and (Data.VarName <> '') then
            for I := 0 to Variables.Count - 1 do
              if Data.VarName = Variables[I].Name then
                Res := Variables[I].Value = Data.VarValue;
        end;
          //execute actions
        if Res then
        begin
          if not Repeating or Data.RepeatActions then
          begin
            for K := 0 to Actions.Count - 1 do
              ExecuteSingleAction(Actions[K]);
            if Data.OSD <> '' then
              ShowOSD(Data.OSD);
            TrayIcon.ChangeIcon(RemoteOnImg.Picture.Icon, ANIMATE_INTERVAL, RemoteOffImg.Picture.Icon);
          end;
          Break;
        end;
      end;
end;

//------------------------------------------------------------------------------

end.

