unit UOSD;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, IniFiles, Math, UMain;

type
  TOSDForm = class(TForm)
    MessageLbl: TLabel;
    Timer: TTimer;
    procedure TimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FVertAlign: TVertAlign;
    FHorizAlign: THorizAlign;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    ClockMode: Boolean;
    TimerInterval: Integer;
    procedure SetAlignement(Vert: TVertAlign; Horiz: THorizAlign);
    procedure ShowMessage(Msg1, Msg2: string; UseDefaultInterval: Boolean = True);
  end;

var
  OSDForm: TOSDForm;

implementation

{$R *.dfm}

//------------------------------------------------------------------------------

procedure TOSDForm.TimerTimer(Sender: TObject);
begin
  if ClockMode then
  begin
    Timer.Interval := 1000;
    ShowMessage(TimeToStr(Time), '', False);
  end
  else begin
    Timer.Enabled := False;
    ShowWindow(Handle, SW_HIDE);
  end;
end;

//------------------------------------------------------------------------------

procedure TOSDForm.SetAlignement(Vert: TVertAlign; Horiz: THorizAlign);
begin
  FVertAlign := Vert;
  FHorizAlign := Horiz;
end;

//------------------------------------------------------------------------------

procedure TOSDForm.ShowMessage(Msg1, Msg2: string; UseDefaultInterval: Boolean = True);
var
  FgHWnd: HWnd;
  StructHandle: HDWP;
begin
  if Msg2 = '' then
    MessageLbl.Caption := ' ' + Msg1 + ' '
  else
    MessageLbl.Caption := ' ' + Msg1 + ' ' + #13#10 + Msg2;
  ClientWidth := MessageLbl.Width;
  ClientHeight := MessageLbl.Height;
  case FVertAlign of
    valTop: Top := 10;
    valCenter: Top := (Screen.Height - ClientHeight - 40) div 2;
    valBottom: Top := Screen.Height - ClientHeight - 50;
  end;
  case FHorizAlign of
    halLeft: Left := 10;
    halCenter: Left := (Screen.Width - ClientWidth) div 2;
    halRight: Left := Screen.Width - ClientWidth - 10;
  end;
  Update;
  Timer.Enabled := False;
  if UseDefaultInterval then
    Timer.Interval := TimerInterval;
  Timer.Enabled := True;

  FgHWnd := GetForegroundWindow;
  StructHandle := BeginDeferWindowPos(1);
  StructHandle := DeferWindowPos(StructHandle, Handle, HWND_TOPMOST, Left, Top, Width, Height, SWP_NOACTIVATE and SWP_NOMOVE and SWP_NOSIZE);
  EndDeferWindowPos(StructHandle);
  ShowWindow(Handle, SW_SHOWNA);
  if (FgHWnd <> NULL_HANDLE) and (FgHWnd <> GetForegroundWindow) then
    SetForegroundWindow(FgHWnd);
end;

//------------------------------------------------------------------------------

procedure TOSDForm.FormCreate(Sender: TObject);
begin
  ClockMode := False;
  with TIniFile.create(InIFileStr) do
  try
    with MessageLbl do
    begin
      Font.Name := ReadString('OSD', 'Font.Name', 'Courier New');
      Font.Color := StringToColor(ReadString('OSD', 'Font.Color', 'clLime'));
      Font.Size := ReadInteger('OSD', 'Font.Size', 48);
      if ReadBool('OSD', 'Font.Bold', False) then
        Font.Style := [fsBold]
      else Font.Style := [];
      if ReadBool('OSD', 'Font.Italic', False) then
        Font.Style := Font.Style + [fsItalic];
      if ReadBool('OSD', 'Font.Underline', False) then
        Font.Style := Font.Style + [fsUnderline];
      Color := StringToColor(ReadString('OSD', 'BgColor', 'clBlack'));
    end;
    Self.Color := MessageLbl.Font.Color;
    BorderWidth := IfThen(ReadBool('OSD', 'Border', True), 3, 0);
    TransparentColorValue := MessageLbl.Color;
    TransparentColor := ReadBool('OSD', 'Transparent', False);
    AlphaBlendValue := ReadInteger('OSD', 'AlphaBlend', 0);
    AlphaBlend := AlphaBlendValue > 0;
    TimerInterval := ReadInteger('OSD', 'ShowTime', 2500);
    SetAlignement(TVertAlign(ReadInteger('OSD', 'VerticalPosition', 0)),
      THorizAlign(ReadInteger('OSD', 'HorizontalPosition', 0)));
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TOSDForm.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.ExStyle := WS_EX_TOPMOST or WS_EX_TOOLWINDOW;
  Params.Style := WS_DISABLED or WS_POPUP;
end;

//------------------------------------------------------------------------------

end.

