unit UVarDef;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IniFiles, UMain;

type
  TVarDefForm = class(TForm)
    VarNameCombo: TComboBox;
    AddVarBtn: TButton;
    RemoveVarBtn: TButton;
    ValuesLb: TListBox;
    ValuesLbl: TLabel;
    NewValueEdt: TEdit;
    RemoveValueBtn: TButton;
    AddValueBtn: TButton;
    OkBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure VarNameComboChange(Sender: TObject);
    procedure AddVarBtnClick(Sender: TObject);
    procedure RemoveVarBtnClick(Sender: TObject);
    procedure NewValueEdtKeyPress(Sender: TObject; var Key: Char);
    procedure AddValueBtnClick(Sender: TObject);
    procedure RemoveValueBtnClick(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    procedure ApplyLanguage(Lang: string);
  end;

var
  VarDefForm: TVarDefForm;

implementation

uses UConfigKeys;

{$R *.dfm}

//------------------------------------------------------------------------------

procedure TVarDefForm.ApplyLanguage(Lang: string);
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.lang')) do
  try
    Self.Caption := ReadString(Lang, 'VariableTitle', 'Variable');
    AddVarBtn.Caption := ReadString(Lang, 'AddVarBtnCaption', 'Add');
    AddValueBtn.Caption := ReadString(Lang, 'AddVarBtnCaption', 'Add');
    RemoveVarBtn.Caption := ReadString(Lang, 'RemoveVarBtnCaption', 'Remove');
    RemoveValueBtn.Caption := ReadString(Lang, 'RemoveVarBtnCaption', 'Remove');
    ValuesLbl.Caption := ReadString(Lang, 'ValuesCaption', 'Values :');
    OkBtn.Caption := ReadString(Lang, 'OkBtnCaption', 'Ok');
  finally
    Free;
  end;
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  ClientHeight := 240;
  ClientWidth := 340;
  VarNameCombo.Clear;
  for I := 0 to Variables.Count - 1 do
    VarNameCombo.Items.Add(Variables[I].Name);
  if VarNameCombo.Items.Count > 0 then
  begin
    VarNameCombo.ItemIndex := 0;
    VarNameComboChange(nil);
  end;
  ApplyLanguage(CurrentLanguageStr);
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.VarNameComboChange(Sender: TObject);
begin
  ValuesLb.Clear;
  if VarNameCombo.ItemIndex > -1 then
    ValuesLb.Items.AddStrings(Variables[VarNameCombo.ItemIndex].Values);
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.AddVarBtnClick(Sender: TObject);
var
  I: Integer;
  VarName: string;
  IsNew: Boolean;
  PVar: PVariable;
begin
  if InputQuery('Add variable', 'Variable name :', VarName) then
  begin
    IsNew := True;
    for I := 0 to Variables.Count - 1 do
      if SameText(Variables[I].Name, VarName) then
      begin
        IsNew := False;
        Break;
      end;
    if IsNew then
    begin
      New(PVar);
      PVar.Name := VarName;
      PVar.Value := -1;
      PVar.Values := TStringList.Create;
      Variables.Add(PVar);
      VarNameCombo.ItemIndex := VarNameCombo.Items.Add(VarName);
      VarNameComboChange(nil);
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.RemoveVarBtnClick(Sender: TObject);
begin
  Variables.Delete(VarNameCombo.ItemIndex);
  VarNameCombo.Items.Delete(VarNameCombo.ItemIndex);
  VarNameComboChange(nil);
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.NewValueEdtKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #13 then
  begin
    Key := #0;
    AddValueBtnClick(nil);
    NewValueEdt.SelectAll;
  end;
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.AddValueBtnClick(Sender: TObject);
var
  I: Integer;
  IsNew: Boolean;
begin
  if (VarNameCombo.ItemIndex > -1) and (NewValueEdt.Text <> '') then
  begin
    IsNew := True;
    for I := 0 to Variables[VarNameCombo.ItemIndex].Values.Count - 1 do
      if SameText(NewValueEdt.Text, Variables[VarNameCombo.ItemIndex].Values[I]) then
      begin
        IsNew := False;
        Break;
      end;
    if IsNew then
    begin
      Variables[VarNameCombo.ItemIndex].Values.Add(NewValueEdt.Text);
      ValuesLb.Items.Add(NewValueEdt.Text);
    end;
  end;
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.RemoveValueBtnClick(Sender: TObject);
begin
  if (VarNameCombo.ItemIndex > -1) and (ValuesLb.ItemIndex > -1) then
  begin
    Variables[VarNameCombo.ItemIndex].Values.Delete(ValuesLb.ItemIndex);
    ValuesLb.Items.Delete(ValuesLb.ItemIndex);
  end;
end;

//------------------------------------------------------------------------------

procedure TVarDefForm.OkBtnClick(Sender: TObject);
var
  I: Integer;
begin
  with ConfigKeysForm.OsdCombo do
  begin
    I := 0;
    while I < Items.Count do
    begin
      while (I < Items.Count) and (Pos('_value]', Items[I]) > 0) do
        Items.Delete(I);
      Inc(I);
    end;
    for I := 0 to Variables.Count - 1 do
      Items.Add(Variables[I].Name + ' = [' + Variables[I].Name + '_value]');
  end;
end;

//------------------------------------------------------------------------------

end.

